searchData={"items":[{"type":"task","title":"mix propcheck","doc":"PropCheck runs property checking as part of ExUnit test and\nstores counter examples of failing properties in order to\nreapply them in the next test run.\n\nThe file name for the counter examples can be configured in `mix.exs`\nin the project configuration as\n\n    propcheck: [counter_example: \"filename\"]\n\nWith `mix propcheck.inspect` you can inspect the found counter examples,\nwith `mix propcheck.clean` the file is deleted afterwards.","ref":"Mix.Tasks.Propcheck.html"},{"type":"function","title":"Mix.Tasks.Propcheck.run/1","doc":"","ref":"Mix.Tasks.Propcheck.html#run/1"},{"type":"task","title":"mix propcheck.clean","doc":"Removes the counter example file of propcheck.","ref":"Mix.Tasks.Propcheck.Clean.html"},{"type":"task","title":"mix propcheck.inspect","doc":"Inspects all counter examples.","ref":"Mix.Tasks.Propcheck.Inspect.html"},{"type":"module","title":"PropCheck","doc":"Provides the macros and functions for property based testing\nusing `proper` as base implementation. `PropCheck` supports many\nfeatures of `PropEr`, but the automated generation of test data\ngenerators is only partially supported due to internal features of\n`PropEr` focusing of Erlang only.","ref":"PropCheck.html"},{"type":"module","title":"Using PropCheck - PropCheck","doc":"To use `PropCheck`, you need to add `use PropCheck` to your Elixir\nfiles. This gives you access to the functions and macros defined\nhere as well as to the `property` macro, defined in\n`PropCheck.Properties.property/4`. To set default options for your\nproperties, you can use `use PropCheck, default_opts: [numtests:\n50]`. `default_opts` can be a list of options defined below or a\nfunction that returns a list of option. In most examples shown\nhere, we directly use the `quickcheck` function, but typically you\nuse the `property` macro instead to define test cases for `ExUnit`.\n\nAlso available are the value generators which are imported directly\nfrom `PropCheck.BasicTypes`.","ref":"PropCheck.html#module-using-propcheck"},{"type":"module","title":"How to write properties - PropCheck","doc":"The simplest properties that PropEr can test consist of a single boolean\nexpression (or a statement block that returns a boolean), which is expected\nto evaluate to `true`. Thus, the test `true` always succeeds, while the test\n`false` always fails (the failure of a property may also be signified by\nthrowing an exception, error or exit. More complex (and useful) properties\ncan be written by wrapping such a boolean expression with one or more of the\nfollowing wrappers:\n\n* `forall/2`\n* `implies/2`\n* `when_fail/2`\n* `trap_exit/1`\n* `conjunction/1`\n* `equals/2`\n\nThere are also multiple wrappers that can be used to collect statistics on\nthe distribution of test data:\n\n* `collect/2`\n* `collect/3`\n* `aggregate/2`\n* `aggregate/3`\n* `classify/3`\n* `measure/3`\n\nA property may also be wrapped with one or more of the following outer-level\nwrappers, which control the behaviour of the testing subsystem. If an\nouter-level wrapper appears more than once in a property, the innermost\ninstance takes precedence.\n\n* `numtests/2`\n* `fails/1`\n* `on_output/2`\n\n`PropCheck` follows the Elixir idioms that for fluent API the first\nparameter flows through a pipeline of functions. Therefore, in `PropCheck`\nthe wrapper functions have the property as first argument allowing to\nuse the `|>` to concatenate wrapper functions. It helps to distinguish\nbetween the property to test and those wrappers which beautify the\nresults or the collection information about the test values. This is a\nsignificant derivation of the API of both, PropEr and QuickCheck.\n\nFor some actual usage examples, see the code in the examples directory, or\ncheck out PropEr's site. The testing modules in the tests directory may also\nbe of interest.","ref":"PropCheck.html#module-how-to-write-properties"},{"type":"module","title":"Program behaviour - PropCheck","doc":"When running in verbose mode (this is the default for `quickcheck`), each successful test\nprints a `.` on screen. If a test fails, a `!` is printed, along with the\nfailing test case (the instances of the types in every `forall`) and the\ncause of the failure, if it was not simply the falsification of the\nproperty.\n\nThen, unless the test was expected to fail, PropEr attempts to produce a\nminimal test case that fails the property in the same way. This process is\ncalled *shrinking*. During shrinking, a `.` is printed for each\nsuccessful simplification of the failing test case. When PropEr reaches its\nshrinking limit or realizes that the instance cannot be shrunk further while\nstill failing the test, it prints the minimal failing test case and failure\nreason and exits.\n\nThe return value of PropEr can be one of the following:\n* `true`: The property held for all valid produced inputs.\n* `false`: The property failed for some input.\n* `{error, type_of_error}`: An error occurred; see the section Errors\n  section for more information.\n\nTo test all properties exported from a module (a property is a 0-arity\nfunction whose name begins with `prop_`), you can use `module/1` or\n`module/2`. This returns a list of all failing properties, represented\nby MFAs. Testing progress is also printed on screen (unless quiet mode is\nactive). The provided options are passed on to each property, except for\n`long_result`, which controls the return value format of the `module`\nfunction itself.","ref":"PropCheck.html#module-program-behaviour"},{"type":"module","title":"Counterexamples - PropCheck","doc":"A counterexample for a property is represented as a list of terms; each such\nterm corresponds to the type in a `forall`. The instances are provided in\nthe same order as the `forall` wrappers in the property, i.e. the instance\nat the head of the list corresponds to the outermost `forall` etc.\nInstances generated inside a failing sub-property of a conjunction are\nmarked with the sub-property's tag.\n\nThe last (simplest) counterexample produced by PropEr during a (failing) run\ncan be retrieved after testing has finished, by running\n`counterexample/0`. When testing a whole module, run\n`counterexamples/0` to get a counterexample for each failing property,\nas a list of `{mfa, counterexample}` tuples. To enable this\nfunctionality, some information has to remain in the process dictionary\neven after PropEr has returned. If, for some reason, you want to completely\nclean up the process dictionary of PropEr-produced entries, run\n`clean_garbage/0`.\n\nCounterexamples can also be retrieved by running PropEr in long-result mode,\nwhere counterexamples are returned as part of the return value.\nSpecifically, when testing a single property under long-result mode\n(activated by supplying the option `:long_result`, or by calling\n`counterexample/1` or `counterexample/2` instead of\n`quickcheck/1` and `quickcheck/2` respectively), PropEr will\nreturn a counterexample in case of failure (instead of simply returning\n`false`). When testing a whole module under long-result mode (activated by\nsupplying the option `:long_result` to `module/2`), PropEr will return\na list of `{mfa(), counterexample}` tuples, one for each failing\nproperty.\n\nYou can re-check a specific counterexample against the property that it\npreviously falsified by running `check/2` or `check/3`. This\nwill return one of the following (both in short- and long-result mode):\n\n* `true`: The property now holds for this test case.\n* `false`: The test case still fails (although not necessarily for the\n  same reason as before).\n* `{error, type_of_error}`: An error occurred - see the section Errors\n  section for more information.\n\nPropEr will not attempt to shrink the input in case it still fails the\nproperty. Unless silent mode is active, PropEr will also print a message on\nscreen, describing the result of the re-checking. Note that PropEr can do\nvery little to verify that the counterexample actually corresponds to the\nproperty that it is tested against.","ref":"PropCheck.html#module-counterexamples"},{"type":"module","title":"Options - PropCheck","doc":"Options can be provided as an extra argument to most testing functions (such\nas `quickcheck/1`). A single option can be written stand-alone, or\nmultiple options can be provided in a list. When two settings conflict, the\none that comes first in the list takes precedence. Settings given inside\nexternal wrappers to a property (see the section on How to write properties)\noverride any conflicting settings provided as options.\n\nThe available options are:\n\n* `:quiet`   Enables quiet mode - no output is printed on screen while PropEr is\n  running.\n* `:verbose`  \n  Enables verbose mode - this is the default mode of operation.\n* `{:to_file, io_device}`  \n  Redirects all of PropEr's output to `io_device`, which should be an\n  IO device associated with a file opened for writing.\n* `{:on_output, output_function}`  \n  PropEr will use the supplied function for all output printing. This\n  function should accept two arguments in the style of `:io.format/2`. \n  **CAUTION:** The above output control options are incompatible with each\n  other.\n* `:long_result`  \n  Enables long-result mode (see the section Counterexamples\n  for details).\n* `{:numtests, positive_number}` or simply `positive_number`  \n  This is equivalent to the `numtests/1` property wrapper. Any\n    `numtests/1` wrappers in the actual property will overwrite this\n  setting.\n* `{:start_size, size}`  \n  Specifies the initial value of the `size` parameter (default is 1), see\n  the documentation of the `PropCheck.BasicTypes` module for details.\n* `{:max_size, size}`  \n  Specifies the maximum value of the `size` parameter (default is 42), see\n  the documentation of the `PropCheck.BasicTypes` module for details.\n* `{:max_shrinks, non_negative_number}`  \n  Specifies the maximum number of times a failing test case should be\n  shrunk before returning. Note that the shrinking may stop before so many\n  shrinks are achieved if the shrinking subsystem deduces that it cannot\n  shrink the failing test case further. Default is 500.\n* `:noshrink`  \n  Instructs PropEr to not attempt to shrink any failing test cases.\n* `{:constraint_tries, positive_number}`  \n  Specifies the maximum number of tries before the generator subsystem\n  gives up on producing an instance that satisfies a `such_that`\n  constraint. Default is 50.\n* `fails`  \n    This is equivalent to the `fails/1` property wrapper.\n* `{:spec_timeout, :infinity |  }`  \n  When testing a spec, PropEr will consider an input to be failing if the\n  function under test takes more than the specified amount of milliseconds\n  to return for that input.\n* `:any_to_integer`  \n    All generated instances of the type `PropCheck.BasicTypes.any/0` will be\n  integers. This is provided as a means to speed up the testing of specs,\n  where `any` is a commonly used type. Remark: PropCheck does not support\n  spec-testing.\n* `{:search_steps, non_negative_number}`  \n  used for targeted properties, see `PropCheck.TargetedPBT`.\n* `{:skip_mfas, [mfa]}`  \n  When checking a module's specs, PropEr will not test the\n  specified MFAs.  Default is []. Remark: PropCheck does not support\n  spec-testing.\n* `{false_positive_mfas, ((mfa(), args::[any], {:fail, result::any} |\n  {:error | :exit | :throw, reason::any}) -> boolean) | :undefined`  \n  When checking a module's spec(s), PropEr will treat a\ncounterexample as a false positive if the user supplied function\nreturns true.  Otherwise, PropEr will treat the counterexample as\nit normally does.  The inputs to the user supplied function are\nthe MFA, the arguments passed to the MFA, and the result returned\nfrom the MFA or an exception with it's reason.  If needed, the\nuser supplied function can call `:erlang.get_stacktrace/0`.  Default\nis `:undefined`. Remark: PropCheck does not support\nspec-testing.","ref":"PropCheck.html#module-options"},{"type":"module","title":"PropCheck Specific Options - PropCheck","doc":"* `{detect_exceptions | boolean}`  \n  PropCheck can attempt to detect exceptions thrown or errors raised during\na testing function. It will output the error and the corresponding stacktrace.","ref":"PropCheck.html#module-propcheck-specific-options"},{"type":"module","title":"Errors - PropCheck","doc":"The following errors may be encountered during testing. The term provided\nfor each error is the error type returned by `quickcheck/2` in case such\nan error occurs. Normally, a message is also printed on screen describing\nthe error.\n\n* `:arity_limit` \n  The random instance generation subsystem has failed to produce\n  a function of the desired arity. Please recompile PropEr with a suitable\n  value for `?MAX_ARITY` (defined in `proper_internal.hrl`). This error\n  should only be encountered during normal operation.\n* `:cant_generate` \n  The random instance generation subsystem has failed to\n  produce an instance that satisfies some `such_that/2` constraint. You\n  should either increase the `:constraint_tries` limit, loosen the failing\n  constraint, or make it non-strict. This error should only be encountered\n  during normal operation.\n* `:cant_satisfy` \n  All the tests were rejected because no produced test case\n  would pass all `implies/2` checks. You should loosen the failing `implies/2`\n  constraint(s). This error should only be encountered during normal\n  operation.\n* `:non_boolean_result` \n  The property code returned a non-boolean result. Please\n  fix your property.\n* `:rejected` \n  Only encountered during re-checking, the counterexample does not\n  match the property, since the counterexample doesn't pass an `implies/2`\n  check.\n* `:too_many_instances` \n  Only encountered during re-checking, the counterexample\n  does not match the property, since the counterexample contains more\n  instances than there are `forall/2`s in the property.\n* `:type_mismatch` \n  The variables' and types' structures inside a `forall/2` don't\n  match. Please check your properties.\n* `{:typeserver, sub_error}` \n  The typeserver encountered an error. The `sub_error` field contains\n  specific information regarding the error.\n* `{:unexpected, result}` \n  A test returned an unexpected result during normal operation. If you\n  ever get this error, it means that you have found a bug in PropEr\n  - please send an error report to the maintainers and remember to include\n  both the failing test case and the output of the program, if possible.\n* `{:unrecognized_option, option}` \n  `option` is not an option that PropEr understands.","ref":"PropCheck.html#module-errors"},{"type":"module","title":"Options Available in the Environment - PropCheck","doc":"* `PROPCHECK_VERBOSE` can be used to enable or disable verbose output globally.\n* `PROPCHECK_DETECT_EXCEPTIONS` can be used to enable or disable exception and error\n  detection globally.","ref":"PropCheck.html#module-options-available-in-the-environment"},{"type":"module","title":"Acknowledgments - PropCheck","doc":"Very much of the documentation is directly taken from the\n`proper` API documentation.","ref":"PropCheck.html#module-acknowledgments"},{"type":"function","title":"PropCheck.aggregate/2","doc":"Same as `collect/2`, but accepts a list of categories under which\nto classify the produced test case.","ref":"PropCheck.html#aggregate/2"},{"type":"function","title":"PropCheck.aggregate/3","doc":"Same as `collect/3`, but accepts a list of categories under which\nto classify the produced test case.","ref":"PropCheck.html#aggregate/3"},{"type":"function","title":"PropCheck.check/3","doc":"Re-checks a specific counterexample `cexm` against the property\n`outer_test` that it previously falsified.","ref":"PropCheck.html#check/3"},{"type":"function","title":"PropCheck.check_spec/2","doc":"Tests the accuracy of an exported function's spec.","ref":"PropCheck.html#check_spec/2"},{"type":"function","title":"PropCheck.check_specs/2","doc":"Tests all exported, `-spec`ed functions of a module `mod` against their spec.","ref":"PropCheck.html#check_specs/2"},{"type":"function","title":"PropCheck.classify/3","doc":"Same as `collect/2`, but can accept both a single category and a\nlist of categories.\n\n`count` is a boolean flag: when `false`, the particular\ntest case will not be counted.","ref":"PropCheck.html#classify/3"},{"type":"function","title":"PropCheck.collect/2","doc":"Specifies that test cases produced by this property should be\ncategorized under the term `category`.\n\nThis field can be an expression or\nstatement block that evaluates to any term. All produced categories are\nprinted at the end of testing (in case no test fails) along with the\npercentage of test cases belonging to each category. Multiple `collect`\nwrappers are allowed in a single property, in which case the percentages for\neach `collect` wrapper are printed separately.","ref":"PropCheck.html#collect/2"},{"type":"function","title":"PropCheck.collect/3","doc":"Same as `collect/2`, but also accepts a fun `printer` to be used\nas the stats printer.","ref":"PropCheck.html#collect/3"},{"type":"function","title":"PropCheck.conjunction/1","doc":"Returns a property that is true only if all of the sub-properties\n`sub_properties` are true.\n\nEach sub-property should be tagged with a distinct atom.\nIf this property fails, each failing sub-property will be reported and saved\ninside the counterexample along with its tag.","ref":"PropCheck.html#conjunction/1"},{"type":"function","title":"PropCheck.counterexample/0","doc":"Retrieves the last (simplest) counterexample produced by PropCheck during\nthe most recent testing run.","ref":"PropCheck.html#counterexample/0"},{"type":"type","title":"PropCheck.counterexample/0","doc":"","ref":"PropCheck.html#t:counterexample/0"},{"type":"function","title":"PropCheck.counterexample/2","doc":"Equivalent to `quickcheck/2`, also accepting a list of options.","ref":"PropCheck.html#counterexample/2"},{"type":"function","title":"PropCheck.counterexamples/0","doc":"Returns a counterexample for each failing property of the most recent\nmodule testing run.","ref":"PropCheck.html#counterexamples/0"},{"type":"macro","title":"PropCheck.delay/1","doc":"Delays the evaluation of `expr`.\n\nRequired for defining recursive generators and similar situations.","ref":"PropCheck.html#delay/1"},{"type":"function","title":"PropCheck.equals/2","doc":"A custom property that evaluates to `true` only if `a === b`, else\nevaluates to `false` and prints `\"A != B\"` on the screen.","ref":"PropCheck.html#equals/2"},{"type":"function","title":"Examples - PropCheck.equals/2","doc":"iex> use PropCheck\n    iex> quickcheck(equals(:ok, :ok))\n    :true\n\n    iex> use PropCheck\n    iex> quickcheck(\n    ...> forall x <- :ok do\n    ...>   equals(:ok, x)\n    ...> end)\n    :true\n\n    iex> use PropCheck\n    iex> quickcheck(\n    ...> forall x <- :not_ok do\n    ...>   equals(:ok, x)\n    ...> end)\n    :false","ref":"PropCheck.html#equals/2-examples"},{"type":"type","title":"PropCheck.error/0","doc":"","ref":"PropCheck.html#t:error/0"},{"type":"type","title":"PropCheck.error_reason/0","doc":"","ref":"PropCheck.html#t:error_reason/0"},{"type":"function","title":"PropCheck.fails/1","doc":"Specifies that we expect the property `property` to fail for some input.\n\nThe property will be considered failing if it passes all the tests.","ref":"PropCheck.html#fails/1"},{"type":"type","title":"PropCheck.false_positive_mfas/0","doc":"","ref":"PropCheck.html#t:false_positive_mfas/0"},{"type":"macro","title":"PropCheck.forall/3","doc":"A property that should hold for all values generated.\n\n    iex> use PropCheck\n    iex> quickcheck(\n    ...> forall n <- nat() do\n    ...>   n >= 0\n    ...> end)\n    true\n\nIf you need more than one generator, collect the generator names\nand the generators definitions in tuples or lists, respectively:\n\n    iex> use PropCheck\n    iex> quickcheck(\n    ...> forall [n, l] <- [nat(), list(nat())] do\n    ...>   n * Enum.sum(l) >= 0\n    ...> end\n    ...>)\n    true\n\nSimilar to `let/2`, multiple types can also be combined in a list of bindings:\n\n    iex> use PropCheck\n    iex> quickcheck(\n    ...> forall [n <- nat(), l <- list(nat())] do\n    ...>   n * Enum.sum(l) >= 0\n    ...> end\n    ...>)\n    true\n\n`forall` allows using `ExUnit` assertions. By default (`:quiet`), no output\nis generated if an assertion does not hold:\n\n    iex> use PropCheck\n    iex> quickcheck(\n    ...> forall n <- nat() do\n    ...>   assert n < 0\n    ...> end\n    ...>)\n    false\n\nUse `:verbose` to enable output on failed `ExUnit` assertions.","ref":"PropCheck.html#forall/3"},{"type":"macro","title":"Setting Verbosity on the Command Line - PropCheck.forall/3","doc":"Apart from setting `:verbose` or `:quiet` explicitly in the source code,\nthe `PROPCHECK_VERBOSE` environment variable can also be used to set the\nverbosity. `PROPCHECK_VERBOSE=1` sets `:verbose`, while `PROPCHECK_VERBOSE=0`\nsets `:quiet`.","ref":"PropCheck.html#forall/3-setting-verbosity-on-the-command-line"},{"type":"macro","title":"PropCheck.implies/2","doc":"A property that is only tested if a condition is true.\n\nThis wrapper only makes sense when in the scope of at least one\n`forall`. The `precondition` field must be a boolean expression or a\nstatement block that returns a boolean. If the precondition evaluates to\n`false` for the variable instances produced in the enclosing `forall`\nwrappers, the test case is rejected (it doesn't count as a failing test\ncase), and `PropCheck` starts over with a new random test case. Also, in\nverbose mode, an `x` is printed on screen.\n\n    iex> use PropCheck\n    iex> require Integer\n    iex> quickcheck(\n    ...> forall n <- nat() do\n    ...>    implies rem(n, 2) == 0, do: Integer.is_even n\n    ...> end\n    ...>)\n    true","ref":"PropCheck.html#implies/2"},{"type":"macro","title":"PropCheck.is_property/1","doc":"","ref":"PropCheck.html#is_property/1"},{"type":"macro","title":"PropCheck.lazy/1","doc":"Mostly internally used macro to create a lazy value for `proper`.\n\nThe parameter `delayed_value` needs to be an already delayed value.","ref":"PropCheck.html#lazy/1"},{"type":"macro","title":"PropCheck.let/2","doc":"Binds a generator to a name for use in another generator.\n\nThe `binding` has the generator syntax `x <- type`.\nTo produce an instance of this type, all appearances of the variables\nin `x` are replaced inside `generator` by their corresponding values in a\nrandomly generated instance of `type`. It's OK for the `gen` part to\nevaluate to a type - in that case, an instance of the inner type is\ngenerated recursively.\n\n    iex> use PropCheck\n    iex> even = let n <- nat() do\n    ...>  n * 2\n    ...> end\n    iex> quickcheck(\n    ...>   forall n <- even do\n    ...>     rem(n, 2) == 0\n    ...>   end)\n    true\n\nIf you require more than one type, put the pairs of variable and type\ninto a list as shown in the example below.\n\n    iex> use PropCheck\n    iex> even_factor = let [n <- nat(), m <- nat()] do\n    ...>  n * m * 2\n    ...> end\n    iex> quickcheck(\n    ...>   forall n <- even_factor do\n    ...>     rem(n, 2) == 0\n    ...>   end)\n    true\n\nYou also can refer to variables declared in the current `let` scope with `^`.\nSuch variables are not allowed to form a dependency cycle.\nYou cannot apply `^` to variables declared outside of the `let` macro.\n\n    iex> use PropCheck\n    iex> non_decreasing =\n    ...> let [m <- integer(^l, :inf), l <- integer(), h <- integer(^m, :inf)] do\n    ...>  {l, m, h}\n    ...> end\n    iex> quickcheck(\n    ...>   forall {l, m, h} <- non_decreasing do\n    ...>     l <= m and m <= h\n    ...>   end)\n    true\n\nSimilar to `forall/2`, multiple types can also be put into tuples or lists:\n\n    iex> use PropCheck\n    iex> even_factor = let {n, m} <- {nat(), nat()} do\n    ...>  n * m * 2\n    ...> end\n    iex> quickcheck(\n    ...>   forall n <- even_factor do\n    ...>     rem(n, 2) == 0\n    ...>   end)\n    true","ref":"PropCheck.html#let/2"},{"type":"macro","title":"PropCheck.let_shrink/2","doc":"A combination of a `let` and a `shrink` macro.\n\nInstances\nare generated by applying a randomly generated list of values inside\n`generator` (just like a `let`, with the added constraint that the\nvariables and types must be provided in a list - alternatively,\n`list_of_types` may be a list or vector type). When shrinking instances\nof such a type, the sub-instances that were combined to produce it are\nfirst tried in place of the failing instance.\n\nOne possible use is shown in the `tree` example. A recursive\ntree generator with an efficient shrinking: pick each of the\nsubtrees in place of the tree that fails the property. `l` and `r`\nare assigned smaller versions of the tree thus achieving a better\n(or more appropriate) shrinking.\n\n      iex> use PropCheck\n      iex> tree_gen = fn (0, _, _) -> :leaf\n      ...>               (s, g, tree) ->\n      ...>      frequency [\n      ...>       {1, tree.(0, g, tree)},\n      ...>       {9, let_shrink([\n      ...>         l <- tree.(div(s, 2), g, tree),\n      ...>         r <- tree.(div(s, 2), g, tree)\n      ...>         ]) do\n      ...>           {:node, g, l, r}\n      ...>         end\n      ...>        }\n      ...>   ]\n      ...> end\n      iex> tree = fn(g) -> sized(s, tree_gen.(s, g, tree_gen)) end\n      iex> quickcheck(\n      ...>   forall t <- tree.(int()) do\n      ...>     t == :leaf or is_tuple(t)\n      ...>   end\n      ...>)\n      true","ref":"PropCheck.html#let_shrink/2"},{"type":"type","title":"PropCheck.long_module_result/0","doc":"","ref":"PropCheck.html#t:long_module_result/0"},{"type":"type","title":"PropCheck.long_result/0","doc":"","ref":"PropCheck.html#t:long_result/0"},{"type":"function","title":"PropCheck.measure/3","doc":"A function that collects numeric statistics on the produced instances.\n\nThe number (or numbers) provided are collected and some statistics over the\ncollected sample are printed at the end of testing (in case no test fails),\nprepended with `title`, which should be an atom or string.","ref":"PropCheck.html#measure/3"},{"type":"function","title":"PropCheck.module/2","doc":"Tests all properties (i.e., all 0-arity functions whose name begins with\n`prop_`) exported from module `mod`","ref":"PropCheck.html#module/2"},{"type":"type","title":"PropCheck.module_result/0","doc":"","ref":"PropCheck.html#t:module_result/0"},{"type":"function","title":"PropCheck.numtests/2","doc":"Specifies the number `N` of tests to run when testing the property\n`property`.\n\nDefault is 100. This function only changes the number of the tests, but\nnot the size of a test.","ref":"PropCheck.html#numtests/2"},{"type":"function","title":"PropCheck.on_output/2","doc":"Specifies an output function `print` to be used by PropCheck for all output\nprinting during the testing of property `property`.\n\nThis wrapper is equivalent to the `on_output` option.","ref":"PropCheck.html#on_output/2"},{"type":"type","title":"PropCheck.outer_test/0","doc":"","ref":"PropCheck.html#t:outer_test/0"},{"type":"type","title":"PropCheck.output_fun/0","doc":"","ref":"PropCheck.html#t:output_fun/0"},{"type":"function","title":"PropCheck.produce/3","doc":"Sample values from a generator `gen`.","ref":"PropCheck.html#produce/3"},{"type":"function","title":"Example - PropCheck.produce/3","doc":"iex> use PropCheck\n    iex> produce(1)\n    {:ok, 1}\n    iex> nat() |> produce() |> Kernel.elem(1) |> is_integer()\n    true\n    iex> nat() |> list() |> produce() |> Kernel.elem(1) |> is_list()\n    true","ref":"PropCheck.html#produce/3-example"},{"type":"macro","title":"PropCheck.property_setup/2","doc":"Adds a setup function to the property which will be called before the\nfirst test.\n\nThis function has to return a finalize function of arity 0,\nwhich should return the atom `:ok`, that will be called after the last test.\nIt is possible to use multiple `property_setup` macros on the same property.","ref":"PropCheck.html#property_setup/2"},{"type":"function","title":"PropCheck.quickcheck/1","doc":"Runs PropEr on the property `outer_test`.","ref":"PropCheck.html#quickcheck/1"},{"type":"function","title":"PropCheck.quickcheck/2","doc":"Same as `quickcheck/1`, but also accepts a list of options.","ref":"PropCheck.html#quickcheck/2"},{"type":"type","title":"PropCheck.result/0","doc":"","ref":"PropCheck.html#t:result/0"},{"type":"type","title":"PropCheck.sample/0","doc":"","ref":"PropCheck.html#t:sample/0"},{"type":"function","title":"PropCheck.sample_shrink/2","doc":"Generates a random instance of Type, of size Size, then shrinks it as far as it goes. The value\nproduced on each step of the shrinking process is printed on the screen.","ref":"PropCheck.html#sample_shrink/2"},{"type":"type","title":"PropCheck.short_module_result/0","doc":"","ref":"PropCheck.html#t:short_module_result/0"},{"type":"type","title":"PropCheck.short_result/0","doc":"","ref":"PropCheck.html#t:short_result/0"},{"type":"macro","title":"PropCheck.shrink/2","doc":"Defines the shrinking of a generator.\n\n`shrink` creates a type whose instances are generated by evaluating the\nstatement block `generator` (this may evaluate to a type, which will\nthen be generated recursively). If an instance of such a type is to be\nshrunk, the generators in `alt_gens` are first run to produce\nhopefully simpler instances of the type. Thus, the generators in the\nsecond argument should be simpler than the default. The simplest ones\nshould be at the front of the list, since those are the generators\npreferred by the shrinking subsystem. Like the main `generator`, the\nalternatives may also evaluate to a type, which is generated recursively.\n\n        iex> use PropCheck\n        iex> quickcheck(\n        ...>   forall n <- shrink(pos_integer(), [0]) do\n        ...>     rem(n, 2) == 0\n        ...>   end)\n        false","ref":"PropCheck.html#shrink/2"},{"type":"type","title":"PropCheck.size/0","doc":"","ref":"PropCheck.html#t:size/0"},{"type":"macro","title":"PropCheck.sized/2","doc":"Changes the maximum size of the generated instances.\n\n`sized` creates a new type, whose instances are produced by replacing all\nappearances of the `size` parameter inside the statement block\n`generator` with the value of the `size` parameter. It's OK for the\n`generator` to return a type - in that case, an instance of the inner\ntype is generated recursively.\n\nAn example for `sized` is shown in the documentation of  `let_shrink/2`.","ref":"PropCheck.html#sized/2"},{"type":"type","title":"PropCheck.stats_printer/0","doc":"","ref":"PropCheck.html#t:stats_printer/0"},{"type":"macro","title":"PropCheck.such_that/2","doc":"This produces a specialization of a generator, encoded as\na binding of form `x <- type` (as in the let macro).\n\nThe specialization of members of `type` that satisfy the constraint\n`condition` - that is, those members for which the function\n`fn(x) -> condition end` returns `true`.\nIf the constraint is very strict - that is, only a small percentage of\ninstances of `type` pass the test - it will take a lot of tries for the\ninstance generation subsystem to randomly produce a valid instance. This\nwill result in slower testing, and testing may even be stopped short, in\ncase the `:constraint_tries` limit is reached (see the \"Options\" section).\n\nIt probably will be more appropriate to generate valid instances of the\nspecialized type using the `let` macro.\nAlso make sure that even small instances can satisfy the constraint, since\nPropEr will **only** increase the size of generated instances when it can\nfind an instance passing the constraint test. If this is not possible, you\ncan instruct PropEr to start at a larger size, by supplying a suitable value\nfor the `:start_size` option (see the \"Options\" section).\n\nIn the second example below, `:start_size` set to 2 is required, because\notherwise PropCheck would first try to generate lists of size 1 (or less),\nbut such a list will never satisfy the _length_ constraint, and after\n`:constraint_tries` limit has been reached, PropCheck would give up.","ref":"PropCheck.html#such_that/2"},{"type":"macro","title":"Examples - PropCheck.such_that/2","doc":"iex> use PropCheck\n    iex> even = such_that n <- nat(), when: rem(n, 2) == 0\n    iex> quickcheck(\n    ...>   forall n <- even do\n    ...>     rem(n, 2) == 0\n    ...>   end)\n    true\n\n    iex> use PropCheck\n    iex> twofer_list = such_that l <- list(nat()), when: length(l) > 1\n    iex> quickcheck(\n    ...>   forall [_ | t] <- twofer_list do\n    ...>     Enum.any?(t)\n    ...>   end, [start_size: 2])\n    true","ref":"PropCheck.html#such_that/2-examples"},{"type":"macro","title":"PropCheck.such_that_maybe/2","doc":"Equivalent to the `such_that` macro, but the constraint `condition`\nis considered non-strict: if the `:constraint_tries` limit is reached, the\ngenerator will just return an instance of `type` instead of failing,\neven if that instance doesn't satisfy the constraint.\n\n    iex> use PropCheck\n    iex> even = such_that_maybe n <- nat(), when: rem(n, 2) == 0\n    iex> quickcheck(\n    ...>   forall n <- even do\n    ...>     rem(n, 2) == 0\n    ...>   end)\n    true","ref":"PropCheck.html#such_that_maybe/2"},{"type":"type","title":"PropCheck.test/0","doc":"","ref":"PropCheck.html#t:test/0"},{"type":"macro","title":"PropCheck.timeout/2","doc":"Signifies that `prop` should be considered failing if it takes more\nthan `time_limit` milliseconds to return.\n\nThe purpose of this wrapper is\nto test code that may hang if something goes wrong. `timeout` cannot\ncontain any more wrappers.\n\n    iex> use PropCheck\n    iex> quickcheck(\n    ...>   timeout(100, forall n <- nat() do\n    ...>     :ok == :timer.sleep(n*100)\n    ...>   end)\n    ...> )\n    false","ref":"PropCheck.html#timeout/2"},{"type":"type","title":"PropCheck.title/0","doc":"","ref":"PropCheck.html#t:title/0"},{"type":"macro","title":"PropCheck.trap_exit/1","doc":"If the code inside `prop` spawns and links to a process that dies\nabnormally, PropEr will catch the exit signal and treat it as a test\nfailure, instead of crashing.\n\n`trap_exit` cannot contain any more wrappers.\n\n    iex> use PropCheck\n    iex> quickcheck(\n    ...>   trap_exit(forall n <- nat() do\n    ...>     # this must fail\n    ...>     _pid = spawn_link(fn() -> n / 0 end)\n    ...>     # wait for arrivial of the dieing linked process signal\n    ...>     :timer.sleep(50)\n    ...>     true #\n    ...>   end)\n    ...> )\n    false","ref":"PropCheck.html#trap_exit/1"},{"type":"type","title":"PropCheck.type/0","doc":"","ref":"PropCheck.html#t:type/0"},{"type":"type","title":"PropCheck.user_opt/0","doc":"","ref":"PropCheck.html#t:user_opt/0"},{"type":"type","title":"PropCheck.user_opts/0","doc":"","ref":"PropCheck.html#t:user_opts/0"},{"type":"macro","title":"PropCheck.when_fail/2","doc":"Execute an action, if the property fails.\n\nThe `action` field should contain an expression or statement block\nthat produces some side-effect (e.g. prints something to the screen).\nIn case this test fails, `action` will be executed. Note that the output\nof such actions is not affected by the verbosity setting of the main\napplication.\n\n    iex> use PropCheck\n    iex> quickcheck(\n    ...>   when_fail(false, IO.puts \"when_fail: Property failed\")\n    ...>)\n    false","ref":"PropCheck.html#when_fail/2"},{"type":"function","title":"PropCheck.with_title/1","doc":"A predefined function that accepts an atom or string and returns a\nstats printing function which is equivalent to the default one, but prints\nthe given title `title` above the statistics.","ref":"PropCheck.html#with_title/1"},{"type":"module","title":"PropCheck.BasicTypes","doc":"This modules contains all basic type generators from PropEr. It is\nautomatically available by `use PropCheck`.","ref":"PropCheck.BasicTypes.html"},{"type":"module","title":"Acknowledgments - PropCheck.BasicTypes","doc":"The functions defined here are delegated to the corresponding\ndefinition `proper_types`. Also most of the documentation is\ncopied over from there.","ref":"PropCheck.BasicTypes.html#module-acknowledgments"},{"type":"function","title":"PropCheck.BasicTypes.any/0","doc":"All Elixir terms (that PropEr can produce).\n\nFor reasons of efficiency, functions are never produced as instances of\nthis type.\n\n**CAUTION:** Instances of this type are expensive to produce, shrink and instance-\ncheck, both in terms of processing time and consumed memory. Only use this\ntype if you are certain that you need it.","ref":"PropCheck.BasicTypes.html#any/0"},{"type":"function","title":"PropCheck.BasicTypes.arity/0","doc":"Arity is a byte value, i.e. `integer(0, 255)`","ref":"PropCheck.BasicTypes.html#arity/0"},{"type":"function","title":"PropCheck.BasicTypes.atom/0","doc":"All atoms.\n\nAll atoms used internally by PropEr start with a `:$` , so\nsuch atoms will never be produced as instances of this type. You should also\nrefrain from using such atoms in your code, to avoid a potential clash.\nInstances shrink towards the empty atom, `:\"\"`.","ref":"PropCheck.BasicTypes.html#atom/0"},{"type":"function","title":"PropCheck.BasicTypes.binary/0","doc":"All binaries.\n\nInstances shrink towards the empty binary, `\"\"`.","ref":"PropCheck.BasicTypes.html#binary/0"},{"type":"function","title":"PropCheck.BasicTypes.binary/1","doc":"All binaries with a byte size of `length`.\n\n`length` must be an Elixir expression that evaluates to a non-negative integer.\nInstances shrink towards binaries of zeroes.","ref":"PropCheck.BasicTypes.html#binary/1"},{"type":"function","title":"PropCheck.BasicTypes.bitstring/0","doc":"All bitstrings.\n\nInstances shrink towards the empty bitstring, `\"\"`.","ref":"PropCheck.BasicTypes.html#bitstring/0"},{"type":"function","title":"PropCheck.BasicTypes.bitstring/1","doc":"All bitstrings with a bit size of `length`.\n\n`length` must be an Elixir expression that evaluates to a non-negative integer.\nInstances shrink towards bitstrings of zeroes.","ref":"PropCheck.BasicTypes.html#bitstring/1"},{"type":"function","title":"PropCheck.BasicTypes.bool/0","doc":"bool is equivalent to `boolean`","ref":"PropCheck.BasicTypes.html#bool/0"},{"type":"function","title":"PropCheck.BasicTypes.boolean/0","doc":"The atoms `true` and `false`. Instances shrink towards `false`.","ref":"PropCheck.BasicTypes.html#boolean/0"},{"type":"function","title":"PropCheck.BasicTypes.byte/0","doc":"Byte values, i.e. `integer(0, 255)`","ref":"PropCheck.BasicTypes.html#byte/0"},{"type":"function","title":"PropCheck.BasicTypes.char/0","doc":"Char values (16 bit for some reason), i.e. `integer(0, 0xffff)`","ref":"PropCheck.BasicTypes.html#char/0"},{"type":"function","title":"PropCheck.BasicTypes.char_list/0","doc":"An Erlang string, i.e. `list(char)`","ref":"PropCheck.BasicTypes.html#char_list/0"},{"type":"function","title":"PropCheck.BasicTypes.choose/2","doc":"choose is equivalent to `integer(low, high)`","ref":"PropCheck.BasicTypes.html#choose/2"},{"type":"function","title":"PropCheck.BasicTypes.default/2","doc":"Adds a default value, `default_value`, to `type`.\n\nThe default serves as a primary shrinking target for instances, while it\nis also chosen by the random instance generation subsystem half the time.","ref":"PropCheck.BasicTypes.html#default/2"},{"type":"function","title":"PropCheck.BasicTypes.elements/1","doc":"elements is equivalent to `union([..])`","ref":"PropCheck.BasicTypes.html#elements/1"},{"type":"function","title":"PropCheck.BasicTypes.exactly/1","doc":"Singleton type consisting only of `value`.\n\n`value` must be an evaluated term. Also written simply as `value`.","ref":"PropCheck.BasicTypes.html#exactly/1"},{"type":"type","title":"PropCheck.BasicTypes.ext_float/0","doc":"Floats extend by infinity","ref":"PropCheck.BasicTypes.html#t:ext_float/0"},{"type":"type","title":"PropCheck.BasicTypes.ext_int/0","doc":"Integers extend by infinity","ref":"PropCheck.BasicTypes.html#t:ext_int/0"},{"type":"type","title":"PropCheck.BasicTypes.ext_non_neg_integer/0","doc":"Non negative integers extend by infinity","ref":"PropCheck.BasicTypes.html#t:ext_non_neg_integer/0"},{"type":"function","title":"PropCheck.BasicTypes.fixed_list/1","doc":"All lists whose i-th element is an instance of the type at index i of\n`list_of_types`. Also written simply as a list of types.","ref":"PropCheck.BasicTypes.html#fixed_list/1"},{"type":"function","title":"PropCheck.BasicTypes.float/0","doc":"All floats, i.e. `float(:inf, :inf)`","ref":"PropCheck.BasicTypes.html#float/0"},{"type":"function","title":"PropCheck.BasicTypes.float/2","doc":"All floats between `low` and `high`, bounds included.\n\n`low` and `high` must be Elixir expressions that evaluate to floats, with\n`Low =< high`. Additionally, `low` and `high` may have the value `:inf`, in\nwhich case they represent minus infinity and plus infinity respectively.\nInstances shrink towards 0.0 if `low =< 0.0 =< high`, or towards the bound\nwith the smallest absolute value otherwise.","ref":"PropCheck.BasicTypes.html#float/2"},{"type":"type","title":"PropCheck.BasicTypes.frequency/0","doc":"","ref":"PropCheck.BasicTypes.html#t:frequency/0"},{"type":"function","title":"PropCheck.BasicTypes.frequency/1","doc":"frequency is equivalent to `weighted_union([..])`","ref":"PropCheck.BasicTypes.html#frequency/1"},{"type":"function","title":"PropCheck.BasicTypes.function0/1","doc":"A function with 0 parameters, i.e. `function(0, ret_type)`","ref":"PropCheck.BasicTypes.html#function0/1"},{"type":"function","title":"PropCheck.BasicTypes.function1/1","doc":"A function with 1 parameter, i.e. `function(1, ret_type)`","ref":"PropCheck.BasicTypes.html#function1/1"},{"type":"function","title":"PropCheck.BasicTypes.function2/1","doc":"A function with 2 parameters, i.e. `function(2, ret_type)`","ref":"PropCheck.BasicTypes.html#function2/1"},{"type":"function","title":"PropCheck.BasicTypes.function3/1","doc":"A function with 3 parameters, i.e. `function(3, ret_type)`","ref":"PropCheck.BasicTypes.html#function3/1"},{"type":"function","title":"PropCheck.BasicTypes.function4/1","doc":"A function with 4 parameters, i.e. `function(4, ret_type)`","ref":"PropCheck.BasicTypes.html#function4/1"},{"type":"function","title":"PropCheck.BasicTypes.function/2","doc":"All pure functions that map instances of `arg_types` to instances of\n`ret_type`.\n\nThe syntax `function(arity, ret_type)` is also acceptable.","ref":"PropCheck.BasicTypes.html#function/2"},{"type":"function","title":"PropCheck.BasicTypes.int/0","doc":"Small integers (bound by the current value of the `size` parameter).\n\nInstances shrink towards `0`.","ref":"PropCheck.BasicTypes.html#int/0"},{"type":"function","title":"PropCheck.BasicTypes.integer/0","doc":"All integers, i.e. `integer(:inf, :inf)`","ref":"PropCheck.BasicTypes.html#integer/0"},{"type":"function","title":"PropCheck.BasicTypes.integer/2","doc":"All integers between `low` and `high`, bounds included.\n\n`low` and `high` must be Elixir expressions that evaluate to integers, with\n`low =< high`. Additionally, `low` and `high` may have the value `:inf`, in\nwhich case they represent minus infinity and plus infinity respectively.\nInstances shrink towards 0 if `low =< 0 =< high`, or towards the bound with\nthe smallest absolute value otherwise.","ref":"PropCheck.BasicTypes.html#integer/2"},{"type":"function","title":"PropCheck.BasicTypes.large_int/0","doc":"Large_int is equivalent to `integer`","ref":"PropCheck.BasicTypes.html#large_int/0"},{"type":"function","title":"PropCheck.BasicTypes.list/0","doc":"List of any types, i.e. `list(any)`","ref":"PropCheck.BasicTypes.html#list/0"},{"type":"function","title":"PropCheck.BasicTypes.list/1","doc":"All lists containing elements of type `elem_type`.\n\nInstances shrink towards the empty list, `[]`.","ref":"PropCheck.BasicTypes.html#list/1"},{"type":"function","title":"PropCheck.BasicTypes.loose_tuple/1","doc":"Tuples whose elements are all of type `elem_type`.\n\nInstances shrink towards the 0-size tuple, `{}`.","ref":"PropCheck.BasicTypes.html#loose_tuple/1"},{"type":"function","title":"PropCheck.BasicTypes.map/2","doc":"A map whose keys are defined by the generator `k` and values by the generator `v`.","ref":"PropCheck.BasicTypes.html#map/2"},{"type":"function","title":"PropCheck.BasicTypes.nat/0","doc":"Small Small non-negative integers (bound by the current value of the `size`\n parameter).\n\nInstances shrink towards `0`.","ref":"PropCheck.BasicTypes.html#nat/0"},{"type":"function","title":"PropCheck.BasicTypes.neg_integer/0","doc":"Negative integers, i.e. `integer(:inf, -1)`","ref":"PropCheck.BasicTypes.html#neg_integer/0"},{"type":"function","title":"PropCheck.BasicTypes.non_empty/1","doc":"This is a predefined constraint that can be applied to random-length\nlist and binary types to ensure that the produced values are never empty.\n\nUse for e.g. `list/0`, `char_list/0`, `binary/0`","ref":"PropCheck.BasicTypes.html#non_empty/1"},{"type":"function","title":"PropCheck.BasicTypes.non_neg_float/0","doc":"Non negative floats, i.e. `float(0.0, inf)`","ref":"PropCheck.BasicTypes.html#non_neg_float/0"},{"type":"function","title":"PropCheck.BasicTypes.non_neg_integer/0","doc":"Non negative integers, i.e. `integer(0, :inf)`","ref":"PropCheck.BasicTypes.html#non_neg_integer/0"},{"type":"function","title":"PropCheck.BasicTypes.noshrink/1","doc":"Creates a new type which is equivalent to `type`, but whose instances\nare never shrunk by the shrinking subsystem.","ref":"PropCheck.BasicTypes.html#noshrink/1"},{"type":"function","title":"PropCheck.BasicTypes.number/0","doc":"Numbers are integers or floats, i.e. `union([integer(), float()])`","ref":"PropCheck.BasicTypes.html#number/0"},{"type":"function","title":"PropCheck.BasicTypes.oneof/1","doc":"oneof is equivalent to `union([..])`","ref":"PropCheck.BasicTypes.html#oneof/1"},{"type":"function","title":"PropCheck.BasicTypes.ordered_list/1","doc":"All sorted lists containing elements of type `elem_type`.\n\nInstances shrink towards the empty list, `[]`.","ref":"PropCheck.BasicTypes.html#ordered_list/1"},{"type":"function","title":"PropCheck.BasicTypes.parameter/1","doc":"Returns the value associated with `parameter`, or `:undefined` in case\n`parameter` is not associated with any value.\n\nAssociation occurs with calling `with_parameter/3` or `with_parameters/2`\nbefore.","ref":"PropCheck.BasicTypes.html#parameter/1"},{"type":"function","title":"PropCheck.BasicTypes.parameter/2","doc":"Returns the value associated with `parameter`, or `default` in case\n`parameter` is not associated with any value.\n\nAssociation occurs with calling `with_parameter/3` or `with_parameters/2`\nbefore.","ref":"PropCheck.BasicTypes.html#parameter/2"},{"type":"function","title":"PropCheck.BasicTypes.pos_integer/0","doc":"Strictly positive integers, i.e. `integer(1, :inf)`","ref":"PropCheck.BasicTypes.html#pos_integer/0"},{"type":"function","title":"PropCheck.BasicTypes.range/2","doc":"A range is equivalent to integers","ref":"PropCheck.BasicTypes.html#range/2"},{"type":"type","title":"PropCheck.BasicTypes.raw_type/0","doc":"The internal representation of a basic type in PropEr","ref":"PropCheck.BasicTypes.html#t:raw_type/0"},{"type":"function","title":"PropCheck.BasicTypes.real/0","doc":"real is equivalent to `float`","ref":"PropCheck.BasicTypes.html#real/0"},{"type":"function","title":"PropCheck.BasicTypes.resize/2","doc":"Overrides the `size` parameter used when generating instances of\n`type` with `new_size`.\n\nHas no effect on size-less types, such as unions.\nAlso, this will not affect the generation of any internal types contained in\n`type`, such as the elements of a list - those will still be generated\nusing the test-wide value of `size`. One use of this function is to modify\ntypes to produce instances that grow faster or slower, like so:\n\n    iex> quickcheck(forall l <- list(integer()) do\n    ...>   length(l) <= 42\n    ...> end)\n    true\n\n    iex> long_list = sized(size, resize(size * 2, list(integer())))\n    iex> really_long = such_that_maybe l <- long_list, when:\n    ...>      length(l) > 42\n    iex> quickcheck(forall l <- really_long do\n    ...>   (length(l) <= 84)\n    ...>   |> measure(\"List length\", length l)\n    ...>   |> collect(length l)\n    ...> end)\n    true\n\nThe above specifies a list type that grows twice as fast as normal lists.","ref":"PropCheck.BasicTypes.html#resize/2"},{"type":"function","title":"PropCheck.BasicTypes.return/1","doc":"return is equivalent to `exactly`","ref":"PropCheck.BasicTypes.html#return/1"},{"type":"function","title":"PropCheck.BasicTypes.shrink_list/1","doc":"A type that generates exactly the list `list`.\n\nInstances shrink towards shorter sublists of the original list.","ref":"PropCheck.BasicTypes.html#shrink_list/1"},{"type":"type","title":"PropCheck.BasicTypes.size/0","doc":"","ref":"PropCheck.BasicTypes.html#t:size/0"},{"type":"function","title":"PropCheck.BasicTypes.term/0","doc":"Term is a synonym for `any`","ref":"PropCheck.BasicTypes.html#term/0"},{"type":"function","title":"PropCheck.BasicTypes.timeout/0","doc":"timeout values, i.e. `union([non_neg_integer() | :infinity])`","ref":"PropCheck.BasicTypes.html#timeout/0"},{"type":"function","title":"PropCheck.BasicTypes.tuple/0","doc":"Tuples of any types, i.e. `loose_tuple(any)`","ref":"PropCheck.BasicTypes.html#tuple/0"},{"type":"function","title":"PropCheck.BasicTypes.tuple/1","doc":"All tuples whose i-th element is an instance of the type at index i of\n`list_of_types`.\n\nAlso written simply as a tuple of types.","ref":"PropCheck.BasicTypes.html#tuple/1"},{"type":"type","title":"PropCheck.BasicTypes.type/0","doc":"The internal representation of a type in PropEr","ref":"PropCheck.BasicTypes.html#t:type/0"},{"type":"function","title":"PropCheck.BasicTypes.union/1","doc":"The union of all types in `list_of_types`.\n\n`list_of_types` can't be empty.\nThe random instance generator is equally likely to choose any one of the\ntypes in `list_of_types`. The shrinking subsystem will always try to shrink an\ninstance of a type union to an instance of the first type in `list_of_types`,\nthus you should write the simplest case first.","ref":"PropCheck.BasicTypes.html#union/1"},{"type":"function","title":"PropCheck.BasicTypes.utf8/0","doc":"utf8-encoded unbounded size binary","ref":"PropCheck.BasicTypes.html#utf8/0"},{"type":"function","title":"PropCheck.BasicTypes.utf8/1","doc":"utf8-encoded bounded upper size binary.","ref":"PropCheck.BasicTypes.html#utf8/1"},{"type":"function","title":"PropCheck.BasicTypes.utf8/2","doc":"Bounded upper size utf8 binary, `codepoint length =< MaxCodePointSize`.\n\nLimiting codepoint size can be useful when applications do not accept full\nunicode range. For example, MySQL in utf8 encoding accepts only 3-byte\nunicode codepoints in VARCHAR fields.\n\nIf unbounded length is needed, use `:inf` as first argument.","ref":"PropCheck.BasicTypes.html#utf8/2"},{"type":"type","title":"PropCheck.BasicTypes.value/0","doc":"","ref":"PropCheck.BasicTypes.html#t:value/0"},{"type":"function","title":"PropCheck.BasicTypes.vector/2","doc":"All lists of length `length` containing elements of type `elem_type`.\n\n`length` must be an Elixir expression that evaluates to a non-negative integer.","ref":"PropCheck.BasicTypes.html#vector/2"},{"type":"function","title":"PropCheck.BasicTypes.weighted_default/2","doc":"A specialization of `default/2`.\n\nParameters `default` and `type` are\nassigned weights to be considered by the random instance generator. The\nshrinking subsystem will ignore the weights and try to shrink using the\ndefault value.","ref":"PropCheck.BasicTypes.html#weighted_default/2"},{"type":"function","title":"PropCheck.BasicTypes.weighted_union/1","doc":"A specialization of `union/1`, where each type in `list_of_types` is\nassigned a frequency.\n\nFrequencies must be Elixir expressions that evaluate to\npositive integers. Types with larger frequencies are more likely to be chosen\nby the random instance generator. The shrinking subsystem will ignore the\nfrequencies and try to shrink towards the first type in the list.","ref":"PropCheck.BasicTypes.html#weighted_union/1"},{"type":"function","title":"PropCheck.BasicTypes.with_parameter/3","doc":"Associates the atom key `parameter` with the value `value` while\ngenerating instances of `type`.","ref":"PropCheck.BasicTypes.html#with_parameter/3"},{"type":"function","title":"PropCheck.BasicTypes.with_parameters/2","doc":"Similar to `with_parameter/3`, but accepts a list of\n`{parameter, value}` pairs.","ref":"PropCheck.BasicTypes.html#with_parameters/2"},{"type":"function","title":"PropCheck.BasicTypes.wunion/1","doc":"weighted_union(FreqChoices)","ref":"PropCheck.BasicTypes.html#wunion/1"},{"type":"behaviour","title":"PropCheck.FSM","doc":"The finite state machine approach for stateful systems, which is closer\nto Erlang's `gen_fsm` model.\n\nThis module defines the `proper_fsm` behaviour, useful for testing\nsystems that can be modeled as finite state machines. That is, a finite\ncollection of named states and transitions between them. `PropCheck.FSM` is\nclosely related to `PropCheck.StateM` and is, in fact, implemented in\nterms of that. Test cases generated using `PropCheck.FSM` will be on precisely\nthe same form as test cases generated using `PropCheck.StateM`. The\ndifference lies in the way the callback modules are specified.\nThe relation between `PropCheck.StateM` and `PropCheck.FSM` is similar\nto the one between `gen_server` and `gen_fsm` in OTP libraries.\n\nDue to name conflicts with functions automatically imported from\n`PropCheck.StateM`, a fully qualified call is needed in order to\nuse the   API functions   of `PropCheck.FSM`.","ref":"PropCheck.FSM.html"},{"type":"behaviour","title":"The states of the finite state machine - PropCheck.FSM","doc":"Following the convention used in `gen_fsm behaviour`, the state is\nseparated into types `t:state_name/0` and some\n`t:state_data/0`. `state_name` is used to denote a state\nof the finite state machine and `state_data` is any relevant information\nthat has to be stored in the model state. States are fully\nrepresented as tuples `{state_name, state_data}`.\n\n`state_name` is usually an atom (i.e. the name of the state), but can also\nbe a tuple. In the latter case, the first element of the tuple must be an\natom specifying the name of the state, whereas the rest of the elements can\nbe arbitrary terms specifying state attributes. For example, when\nimplementing the fsm of an elevator which can reach n different floors, the\n`state_name` for each floor could be `{:floor, k}, 1 <= k <= n`.\n\n`state_data` can be an arbitrary term, but is usually a record.","ref":"PropCheck.FSM.html#module-the-states-of-the-finite-state-machine"},{"type":"behaviour","title":"Transitions between states - PropCheck.FSM","doc":"A transition `t:transition/0` is represented as a tuple\n`{target_state, {:call, m, f, a}}`. This means that performing the specified\nsymbolic call at the current state of the fsm will lead to `target_state`.\nThe atom `:history` can be used as `target_state` to denote that a transition\ndoes not change the current state of the fsm.","ref":"PropCheck.FSM.html#module-transitions-between-states"},{"type":"behaviour","title":"The callback functions - PropCheck.FSM","doc":"The following functions must be exported from the callback module\nimplementing the finite state machine:\n\n* `c:initial_state/0`\n* `c:initial_data/0`\n* `c:precondition/4`\n* `c:postcondition/5`\n* `c:next_state_data/5`\n* `c:weight/3`\n\nIn addition to these functions, we also need functions for each\nstate:\n\n* `state_name(s::state_data) ::[transition]`\n   There should be one instance of this function for each reachable\n  state `state_name` of the finite state machine. In case `state_name` is a\n  tuple the function takes a different form, described just below. The\n  function returns a list of possible transitions (`t:transition/0` )\n  from the current state.\n\n  At command generation time, the instance of this function with the same\n  name as the current state's name is called to return the list of possible\n  transitions. Then, PropEr will randomly choose a transition and,\n  according to that, generate the next symbolic call to be included in the\n  command sequence. However, before the call is actually included, a\n  precondition that might impose constraints on `state_data` is checked.\n\n  Note also that PropEr detects transitions that would raise an exception\n  of class ` ` at generation time (not earlier) and does not choose\n  them. This feature can be used to include conditional transitions that\n  depend on the `t:state_data/0`.\n* `state_name(attr1::any, ..., attrN::any,\n                s::type state_data) :: [transition]`\n   There should be one instance of this function for each reachable state\n  `{state_name, attr1, ..., attrN}` of the finite state machine. The function\n  has similar behaviour to `state_name/1`, described above.","ref":"PropCheck.FSM.html#module-the-callback-functions"},{"type":"behaviour","title":"The property used - PropCheck.FSM","doc":"This is an example of a property that can be used to test a\nfinite state machine specification. It expects a `cleanup` function\nthat takes care of removing all artifacts created during tests to\nenable a clean start for each test case execution.\n\n    property \"fsm\" do\n       forall cmds <- commands(__MODULE__) do\n         {_history, _state, result} = run_commands(__MODULE__, cmds)\n         cleanup()\n         result == :ok\n      end\n    end","ref":"PropCheck.FSM.html#module-the-property-used"},{"type":"type","title":"PropCheck.FSM.cmd_result/0","doc":"","ref":"PropCheck.FSM.html#t:cmd_result/0"},{"type":"type","title":"PropCheck.FSM.command/0","doc":"","ref":"PropCheck.FSM.html#t:command/0"},{"type":"type","title":"PropCheck.FSM.command_list/0","doc":"","ref":"PropCheck.FSM.html#t:command_list/0"},{"type":"function","title":"PropCheck.FSM.command_names/1","doc":"","ref":"PropCheck.FSM.html#command_names/1"},{"type":"function","title":"PropCheck.FSM.commands/1","doc":"A special PropEr type which generates random command sequences,\naccording to a finite state machine specification.\n\nThe function takes as\ninput the name of a callback module, which contains the fsm specification.\nThe initial state is computed by\n`{mod.initial_state/0, mod:initial_state_data/0}`.","ref":"PropCheck.FSM.html#commands/1"},{"type":"function","title":"PropCheck.FSM.commands/2","doc":"Similar to `commands/1`, but generated command sequences always\nstart at a given state.\n\nIn this case, the first command is always\n`{:init, initial_state = {name, data}}` and is used to correctly initialize the\nstate every time the command sequence is run (i.e. during normal execution,\nwhile shrinking and when checking a counterexample).","ref":"PropCheck.FSM.html#commands/2"},{"type":"type","title":"PropCheck.FSM.fsm_result/0","doc":"","ref":"PropCheck.FSM.html#t:fsm_result/0"},{"type":"type","title":"PropCheck.FSM.fsm_state/0","doc":"","ref":"PropCheck.FSM.html#t:fsm_state/0"},{"type":"type","title":"PropCheck.FSM.history/0","doc":"","ref":"PropCheck.FSM.html#t:history/0"},{"type":"callback","title":"PropCheck.FSM.initial_data/0","doc":"Specifies what the state data should initially contain. Its result\nshould be deterministic.","ref":"PropCheck.FSM.html#c:initial_data/0"},{"type":"callback","title":"PropCheck.FSM.initial_state/0","doc":"Specifies the initial state of the finite state machine. As with\n`c:PropCheck.StateM.initial_state/0`, its result should be deterministic.","ref":"PropCheck.FSM.html#c:initial_state/0"},{"type":"type","title":"PropCheck.FSM.mod_name/0","doc":"","ref":"PropCheck.FSM.html#t:mod_name/0"},{"type":"function","title":"PropCheck.FSM.more_commands/2","doc":"","ref":"PropCheck.FSM.html#more_commands/2"},{"type":"callback","title":"PropCheck.FSM.next_state_data/5","doc":"Similar to `c:PropCheck.StateM.next_state/3`. Specifies how the\ntransition from `from` to `target` triggered by `call` affects the\n`state_data`. `res` refers to the result of `call` and can be either\nsymbolic or dynamic.","ref":"PropCheck.FSM.html#c:next_state_data/5"},{"type":"callback","title":"PropCheck.FSM.postcondition/5","doc":"Similar to `c:PropCheck.StateM.postcondition/3`. Specifies the\npostcondition that should hold about the result `res` of the evaluation\nof `call`.","ref":"PropCheck.FSM.html#c:postcondition/5"},{"type":"callback","title":"PropCheck.FSM.precondition/4","doc":"Similar to `c:PropCheck.StateM.precondition/2`.\n\nSpecifies the\nprecondition that should hold about `state_data` so that `call` can be\nincluded in the command sequence. In case precondition doesn't hold, a\nnew transition is chosen using the appropriate `state_name/1` generator.\n\nIt is possible for more than one transitions to be triggered by the same\nsymbolic call and lead to different target states. In this case, at most\none of the target states may have a true precondition. Otherwise, PropEr\nwill not be able to detect which transition was chosen and an exception\nwill be raised.","ref":"PropCheck.FSM.html#c:precondition/4"},{"type":"type","title":"PropCheck.FSM.result/0","doc":"","ref":"PropCheck.FSM.html#t:result/0"},{"type":"function","title":"PropCheck.FSM.run_commands/2","doc":"Evaluates a given symbolic command sequence `cmds` according to the\nfinite state machine specified in `mod`.\n\nThe result is a triple of the form `{history, fsm_state, result}`,\nsimilar to `PropCheck.StateM.run_commands/2`.","ref":"PropCheck.FSM.html#run_commands/2"},{"type":"function","title":"PropCheck.FSM.run_commands/3","doc":"Similar to `run_commands/2`, but also accepts an environment\nused for symbolic variable evaluation, exactly as described in\n`PropCheck.StateM.run_commands/3`.","ref":"PropCheck.FSM.html#run_commands/3"},{"type":"type","title":"PropCheck.FSM.state_data/0","doc":"","ref":"PropCheck.FSM.html#t:state_data/0"},{"type":"type","title":"PropCheck.FSM.state_name/0","doc":"","ref":"PropCheck.FSM.html#t:state_name/0"},{"type":"function","title":"PropCheck.FSM.state_names/1","doc":"Extracts the names of the states from a given command execution history.\n\nIt is useful in combination with functions such as `PropCheck.aggregate/2`\nin order to collect statistics about state transitions during command\nexecution.","ref":"PropCheck.FSM.html#state_names/1"},{"type":"type","title":"PropCheck.FSM.symbolic_call/0","doc":"","ref":"PropCheck.FSM.html#t:symbolic_call/0"},{"type":"type","title":"PropCheck.FSM.symbolic_var/0","doc":"","ref":"PropCheck.FSM.html#t:symbolic_var/0"},{"type":"type","title":"PropCheck.FSM.transition/0","doc":"","ref":"PropCheck.FSM.html#t:transition/0"},{"type":"callback","title":"PropCheck.FSM.weight/3","doc":"This is an optional callback. When it is not defined (or not exported),\ntransitions are chosen with equal probability. When it is defined, it\nassigns an integer weight to transitions from `from` to `target`\ntriggered by symbolic call `call`. In this case, each transition is chosen\nwith probability proportional to the weight assigned.","ref":"PropCheck.FSM.html#c:weight/3"},{"type":"behaviour","title":"PropCheck.Instrument","doc":"Provides functions and macros for instrument byte code with additional yields and\nother constructs to ease testing of concurrent programs and state machines.","ref":"PropCheck.Instrument.html"},{"type":"behaviour","title":"Why is instrumentation important? - PropCheck.Instrument","doc":"The Erlang scheduler is relatively predictable and stable with regard to pre-emptive\nscheduling. This means that every run has more or the less the same amount of\nvirtual machine instructions before a switch to another process happens. These\nprocess switches are required to reveal any concurrency bugs. A simple way to\nprovoke more process switches are calls to `:erlang.yield()` which gives the scheduler\nthe possibility to switch early on to another process. It is not defined if\nthe scheduler reacts on this hint, but it often does and allows for more\nunpredictable schedules revealing more concurrency bugs.\n\nThe usual advice is to sprinkle the code under test with manually added\ncalls to `:erlang.yield()`, but this is a daunting task. Additionally, you\nneed to remove this additional code before production use.","ref":"PropCheck.Instrument.html#module-why-is-instrumentation-important"},{"type":"behaviour","title":"The instrumentation - PropCheck.Instrument","doc":"The functions in this module automate the instrumentation immediately before\nrunning the tests. We instrument call to \"interesting\" functions of the Erlang\nand Elixir ecosystem, e.g. calls to `GenServer` or `ets` tables. We do this by examining\nthe byte code, checking each function call, and if we found some interesting call target,\nwe add a call to `:erlang.yield()` immediately before. This is what the\n`PropCheck.YieldInstrumenter` module provides. It implements the behaviour `Instrument`,\nwhich requires the implementation of two callbacks `c:handle_function_call/1` and\n`c:is_instrumentable_function/2`. After instrumentation, the code reloading mechanism of\nthe Erlang VM enables the new code and the tests can run.","ref":"PropCheck.Instrument.html#module-the-instrumentation"},{"type":"behaviour","title":"Typical usage - PropCheck.Instrument","doc":"To ensure instrumentation before running the tests, you implement the `setup_all` macro\nof `ExUnit`:\n\n    setup_all do\n      Instrument.instrument_module(Cache, YieldInstrumenter)\n      :ok # no update of a context\n    end\n\nIn this example, we instrument only a specific module. You can also instrument\nall modules of an application by calling `Instrument.instrument_app(:my_app_under_test, YieldInstrumenter)`.","ref":"PropCheck.Instrument.html#module-typical-usage"},{"type":"behaviour","title":"Implementing your own instrumenter - PropCheck.Instrument","doc":"For implementing your own instrumenter, you need to get acquainted with the Erlang\nAbstract Form (EAF), which is the internal abstract syntax tree available to the Erlang VM at runtime.\nThis format is quite different from the Elixir AST, in particular it has not the regular form but\nconsists of many different structures. This requires a lot of cases to be handled for analyizing\nthe AST. Little helpers for encoding the instrumented code is provided by `encode_call/1` and\n`encode_value/1` as well as by `prepend_call/2`. For debugging and revealing the structure of\na specific EAF, you can use `print_fun/1`.","ref":"PropCheck.Instrument.html#module-implementing-your-own-instrumenter"},{"type":"function","title":"PropCheck.Instrument.call_yield/0","doc":"Encodes a call to `:erlang.yield()` as Erlang Astract Form.","ref":"PropCheck.Instrument.html#call_yield/0"},{"type":"function","title":"PropCheck.Instrument.compile_module/3","doc":"Compiles the abstract code of a module and loads it immediately into\nthe VM.","ref":"PropCheck.Instrument.html#compile_module/3"},{"type":"function","title":"PropCheck.Instrument.encode_call/1","doc":"Enocdes a call given as tuple `{m, f, a}` as Erlang Abstract Form.","ref":"PropCheck.Instrument.html#encode_call/1"},{"type":"function","title":"PropCheck.Instrument.encode_call/3","doc":"Encodes a call to `m.f.(a)` as Erlang Abstract Form.","ref":"PropCheck.Instrument.html#encode_call/3"},{"type":"function","title":"PropCheck.Instrument.encode_value/1","doc":"Encodes a value as Erlang Astract Form.","ref":"PropCheck.Instrument.html#encode_value/1"},{"type":"type","title":"PropCheck.Instrument.erl_ast_atom_type/0","doc":"The type for a node in the Erlang Abstract Form encoding an atom value","ref":"PropCheck.Instrument.html#t:erl_ast_atom_type/0"},{"type":"type","title":"PropCheck.Instrument.erl_ast_block/0","doc":"Type type for a block of expression in Erlang Abstract Form","ref":"PropCheck.Instrument.html#t:erl_ast_block/0"},{"type":"type","title":"PropCheck.Instrument.erl_ast_remote_call/0","doc":"The type for a remote call in Erlang Abstract Form","ref":"PropCheck.Instrument.html#t:erl_ast_remote_call/0"},{"type":"function","title":"PropCheck.Instrument.get_forms_of_module/1","doc":"Retrieves the abstract code, i.e. the list of forms, of the given\nmodule as found in the code server.","ref":"PropCheck.Instrument.html#get_forms_of_module/1"},{"type":"callback","title":"PropCheck.Instrument.handle_function_call/1","doc":"Handle the instrumentation of a (remote) function call. Must return a\nvalid expression in Erlang Abstract Form.","ref":"PropCheck.Instrument.html#c:handle_function_call/1"},{"type":"function","title":"PropCheck.Instrument.instrument_app/2","doc":"Instruments all modules of an entire OTP application.","ref":"PropCheck.Instrument.html#instrument_app/2"},{"type":"function","title":"PropCheck.Instrument.instrument_module/2","doc":"Takes the object code of the module, instruments it and update the module\nin the code server with instrumented byte code.","ref":"PropCheck.Instrument.html#instrument_module/2"},{"type":"function","title":"PropCheck.Instrument.instrumentable_function/2","doc":"Checks if the given function is a candidate for instrumentation, i.e. does something\ninteresting with respect to concurrency. Examples are process handling, handling\nof shared state or sending and receiving messages.","ref":"PropCheck.Instrument.html#instrumentable_function/2"},{"type":"callback","title":"PropCheck.Instrument.is_instrumentable_function/2","doc":"A callback to decide if the function `mod:fun` with any arity is a candidate\nfor instrumentation. The default implementation is simply calling\n`instrumentable_function/2`.","ref":"PropCheck.Instrument.html#c:is_instrumentable_function/2"},{"type":"function","title":"PropCheck.Instrument.is_instrumented?/1","doc":"Checks if the code is already instrumented. If not, returns `false` otherwise returns `true`","ref":"PropCheck.Instrument.html#is_instrumented?/1"},{"type":"function","title":"PropCheck.Instrument.prepend_call/2","doc":"Prepends the call to `to_be_wrapped_call` by a call to `new_call`.\nThe result of `new_call` is ignored.\n\nAll arugments and return values are in Erlang Astract Form.","ref":"PropCheck.Instrument.html#prepend_call/2"},{"type":"function","title":"PropCheck.Instrument.print_fun/2","doc":"Debugging aid for analyzing code generations. Prints the restructered Erlang code of function\n`fun` in module `mod`. We use Erlang code here, because Elixir source code cannot generated from\nthe byte code format due to macros, which change the compilation process too heavily.","ref":"PropCheck.Instrument.html#print_fun/2"},{"type":"module","title":"PropCheck.OutputAgent","doc":"An agent to gather unique PropCheck-internal output from tests.","ref":"PropCheck.OutputAgent.html"},{"type":"function","title":"PropCheck.OutputAgent.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"PropCheck.OutputAgent.html#child_spec/1"},{"type":"function","title":"PropCheck.OutputAgent.close/1","doc":"Stop the agent and retrieve the output.","ref":"PropCheck.OutputAgent.html#close/1"},{"type":"function","title":"PropCheck.OutputAgent.put/2","doc":"Put new output to the agent.","ref":"PropCheck.OutputAgent.html#put/2"},{"type":"function","title":"PropCheck.OutputAgent.start_link/0","doc":"Start the agent.","ref":"PropCheck.OutputAgent.html#start_link/0"},{"type":"module","title":"PropCheck.Properties","doc":"This module defines the `property/4` and `property/1` macros. It is automatically available\nby `use PropCheck`.","ref":"PropCheck.Properties.html"},{"type":"macro","title":"PropCheck.Properties.property/1","doc":"Defines a not yet implemented property.\n\nThis convenient macro provides a property which will always flunk. It resembles\nthe `test/1` macro from ExUnit. Similarly to `ExUnit`, it also tags the test with\n`:not_implemented`, allowing to filter it when running `mix test`.","ref":"PropCheck.Properties.html#property/1"},{"type":"macro","title":"Example - PropCheck.Properties.property/1","doc":"property \"This property will be implemented in the future\"","ref":"PropCheck.Properties.html#property/1-example"},{"type":"macro","title":"PropCheck.Properties.property/4","doc":"Defines a property as part of an ExUnit test.\n\nThe property macro takes at minimum a name and a `do`-block containing\nthe code of the property to be tested. The property code is encapsulated\nas an `ExUnit` test case of category `property`, which is released as\npart of Elixir 1.3 and allows a nice mix of regular unit test and property\nbased testing. This is the reason for the third parameter taking an\nenvironment of variables defined in a test setup function. In `ExUnit`, this\nis referred to as a test's \"context\".\n\nThe second parameter sets options for Proper (see `PropCheck` ). The default\nis `:quiet` such that execution during ExUnit runs are silent, as normal\nunit tests are. You can change it e.g. to `:verbose` or setting the\nmaximum size of the test data generated or what ever may be helpful. For\nseeing the result of wrapper functions `PropCheck.aggregate/2` etc., the\nverbose mode is required.","ref":"PropCheck.Properties.html#property/4"},{"type":"macro","title":"Counter Examples - PropCheck.Properties.property/4","doc":"If a property fails, the counter example is in a file. The next time this\nproperty is checked again, only the counter example is used to ensure that\nthe property now behaves correctly. Additionally, a property with an existing\ncounter example is embellished with the tag `failing_prop`. You can skip all\nother tests and property by running `mix test --only failing_prop`. In this case\nonly the properties with counter example are run. Another option is to use\nthe `--stale` option of `ExUnit` to reduce the amount of tests and properties\nwhile fixing the code tested by a property.\n\nAfter a property was ran successfully against a previous counter example, PropCheck will\nrun the property again to check if other counter examples can be found.","ref":"PropCheck.Properties.html#property/4-counter-examples"},{"type":"macro","title":"Disable Storing Counter Examples - PropCheck.Properties.property/4","doc":"Storing counter examples can be disabled using the `:store_counter_example` tag. This\ncan be done in three different scopes: module-wide scope, describe-wide scope or for\na single property.\n\n**NOTE** that this facility is meant for properties which cannot run with a value generated\nin a previous test run. This should usually not be the case, and `:store_counter_example`\nshould only be used after careful consideration.\n\nDisable for all properties in a module:\n\n```\ndefmodule Test do\n  # ...\n  @moduletag store_counter_example: false\n  #...\nend\n```\n\nDisable for all properties in a describe block:\n\n```\ndefmodule Test do\n  # ...\n  describe \"describe block\" do\n    @describetag store_counter_example: false\n    # ...\n  end\nend\n```\n\nDisable for a single property:\n\n```\n@tag store_counter_example: false\nproperty \"a property\" do\n  # ...\nend\n```","ref":"PropCheck.Properties.html#property/4-disable-storing-counter-examples"},{"type":"behaviour","title":"PropCheck.StateM","doc":"This module defines the `:proper_statem` behaviour, useful for testing\nstateful reactive systems whose internal state and side-effects are\nspecified via an abstract state machine. Given a callback module\nimplementing the `:proper_statem` behaviour (i.e. defining an abstract state\nmachine of the system under test), PropEr can generate random symbolic\nsequences of calls to that system.\n\nAs a next step, generated symbolic calls are actually performed, while\nmonitoring the system's responses to ensure it behaves as expected. Upon\nfailure, the shrinking mechanism attempts to find a minimal sequence of\ncalls provoking the same error.","ref":"PropCheck.StateM.html"},{"type":"behaviour","title":"The role of commands - PropCheck.StateM","doc":"Test cases generated for testing a stateful system are lists of symbolic API\ncalls to that system. Symbolic representation has several benefits, which\nare listed here in increasing order of importance:\n\n * Generated test cases are easier to read and understand.\n * Failing test cases are easier to shrink.\n * The generation phase is side-effect free and this results in\n  repeatable test cases, which is essential for correct shrinking.\n\nSince the actual results of symbolic calls are not known at generation time,\nwe use symbolic variables of type `t:symbolic_var/0` to refer to them.\nA command of type `t:command/0` is a symbolic term, used to bind a symbolic\nvariable to the result of a symbolic call. For example:\n\n    [{:set, {:var, 1}, {:call, :erlang, :put, [:a, 42]}},\n    {:set, {:var, 2}, {:call, :erlang, :erase, [:a]}},\n    {:set, {:var, 3}, {:call, :erlang, :put, [:b, {:var, 2}]}}]\n\nis a command sequence that could be used to test the process dictionary.\nIn this example, the first call stores the pair `{:a, 42}` in the process\ndictionary, while the second one deletes it. Then, a new pair `{:b, {:var, 2}}`\nis stored. `{:var, 2}` is a symbolic variable bound to the result of\n`:erlang.erase/1`. This result is not known at generation time, since none of\nthese operations is performed at that time. After evaluating the command\nsequence at runtime, the process dictionary will eventually contain the\npair `{:b, 42}`.","ref":"PropCheck.StateM.html#module-the-role-of-commands"},{"type":"behaviour","title":"The abstract model-state - PropCheck.StateM","doc":"In order to be able to test impure code, we need a way to track its\ninternal state (at least the useful part of it). To this end, we use an\nabstract state machine representing the possible configurations of the\nsystem under test. When referring to the *model state*, we mean the\nstate of the abstract state machine. The *model state* can be either\nsymbolic or dynamic:\n\n * During command generation, we use symbolic variables to bind the\n   results of symbolic calls. Therefore, the model state might\n   (and usually does) contain symbolic variables and/or symbolic calls, which\n   are necessary to operate on symbolic variables. Thus, we refer to it as\n   symbolic state. For example, assuming that the internal state of the\n   process dictionary is modeled as a proplist, the model state after\n   generating the previous command sequence will be `[b: {:var, 2}}]`.\n * During runtime, symbolic calls are evaluated and symbolic variables are\n   replaced by their corresponding real values. Now we refer to the state as\n   dynamic state. After running the previous command sequence, the model state\n  will be `[b: 42]`.","ref":"PropCheck.StateM.html#module-the-abstract-model-state"},{"type":"behaviour","title":"The callback functions - PropCheck.StateM","doc":"The following functions must be exported from the callback module\nimplementing the abstract state machine:\n\n * `c:initial_state/0`\n * `c:command/1`\n * `c:precondition/2`\n * `c:postcondition/3`\n * `c:next_state/3`","ref":"PropCheck.StateM.html#module-the-callback-functions"},{"type":"behaviour","title":"The property used - PropCheck.StateM","doc":"Each test consists of two phases:\n\n * As a first step, PropEr generates random symbolic command sequences\n  deriving information from the callback module implementing the abstract\n  state machine. This is the role of `commands/1` generator.\n * As a second step, command sequences are executed so as to check that\n  the system behaves as expected. This is the role of\n  `run_commands/2`, a function that evaluates a symbolic command\n  sequence according to an abstract state machine specification.\n\nThese two phases are encapsulated in the following property, which can be\nused for testing the process dictionary:\n\n    def prop_pdict() do\n      forall cmds <- commands(__MODULE__) do\n        {_history, _state, result} = run_commands(__MODULE__, cmds)\n        cleanup()\n        result == ok\n      end\n    end\n\nWhen testing impure code, it is very important to keep each test\nself-contained. For this reason, almost every property for testing stateful\nsystems contains some clean-up code. Such code is necessary to put the\nsystem in a known state, so that the next test can be executed\nindependently from previous ones.","ref":"PropCheck.StateM.html#module-the-property-used"},{"type":"behaviour","title":"Parallel testing - PropCheck.StateM","doc":"After ensuring that a system's behaviour can be described via an abstract\nstate machine when commands are executed sequentially, it is possible to\nmove to parallel testing. The same state machine can be used to generate\ncommand sequences that will be executed in parallel to test for race\nconditions. A parallel test case (`t:parallel_testcase/0` ) consists of\na sequential and a parallel component. The sequential component is a\ncommand sequence that is run first to put the system in a random state.\nThe parallel component is a list containing 2 command sequences to be\nexecuted in parallel, each of them in a separate newly-spawned process.\n\nGenerating parallel test cases involves the following actions. Initially,\nwe generate a command sequence deriving information from the abstract\nstate machine specification, as in the case of sequential statem testing.\nThen, we parallelize a random suffix (up to 12 commands) of the initial\nsequence by splitting it into 2 subsequences that will be executed\nconcurrently. Limitations arise from the fact that each subsequence should\nbe a *valid* command sequence (i.e. all commands should satisfy\npreconditions and use only symbolic variables bound to the results of\npreceding calls in the same sequence). Furthermore, we apply an additional\ncheck: we have to ensure that preconditions are satisfied in all possible\ninterleavings of the concurrent tasks. Otherwise, an exception might be\nraised during parallel execution and lead to unexpected (and unwanted) test\nfailure. In case these constraints cannot be satisfied for a specific test\ncase, the test case will be executed sequentially. Then an `f` is printed\non screen to inform the user. This usually means that preconditions need\nto become less strict for parallel testing to work.\n\nAfter running a parallel test case, PropEr uses the state machine\nspecification to check if the results observed could have been produced by\na possible serialization of the parallel component. If no such serialization\nis possible, then an atomicity violation has been detected. In this case,\nthe shrinking mechanism attempts to produce a counterexample that is minimal\nin terms of concurrent operations. Properties for parallel testing are very\nsimilar to those used for sequential testing.\n\n    def prop_parallel_testing() do\n       forall testcase <- parallel_commands(__MODULE__) do\n          {_sequential, _parallel, result} = run_parallel_commands(__MODULE__, testcase),\n          cleanup(),\n          result == :ok\n       end\n    end\n\nPlease note that the actual interleaving of commands of the parallel\ncomponent depends on the Erlang scheduler, which is too deterministic.\nFor PropEr to be able to detect race conditions, the code of the system\nunder test should be instrumented with `erlang:yield/0` calls to the\nscheduler.","ref":"PropCheck.StateM.html#module-parallel-testing"},{"type":"behaviour","title":"Acknowledgments - PropCheck.StateM","doc":"Very much of the documentation is immediately taken from the\n`proper` API documentation.","ref":"PropCheck.StateM.html#module-acknowledgments"},{"type":"type","title":"PropCheck.StateM.command/0","doc":"A value of type `command` denotes the execution of a symbolic command and\nstoring its result in a symbolic variable.","ref":"PropCheck.StateM.html#t:command/0"},{"type":"callback","title":"PropCheck.StateM.command/1","doc":"Generates a symbolic call to be included in the command sequence,\ngiven the current state `s` of the abstract state machine.\n\nHowever,\nbefore the call is actually included, a precondition is checked. This\nfunction will be repeatedly called to produce the next call to be\nincluded in the test case.","ref":"PropCheck.StateM.html#c:command/1"},{"type":"type","title":"PropCheck.StateM.command_list/0","doc":"A sequence of commands.","ref":"PropCheck.StateM.html#t:command_list/0"},{"type":"function","title":"PropCheck.StateM.command_names/1","doc":"Extracts the names of the commands from a given command sequence, in\nthe form of MFAs.\n\nIt is useful in combination with functions such as\n`PropCheck.aggregate/2` in order to collect statistics about command\nexecution.","ref":"PropCheck.StateM.html#command_names/1"},{"type":"function","title":"PropCheck.StateM.commands/1","doc":"A special PropEr type which generates random command sequences,\naccording to an abstract state machine specification.\n\nThe function takes as\ninput the name of a callback module, which contains the state machine\nspecification. The initial state is computed by `mod:initial_state/0`.","ref":"PropCheck.StateM.html#commands/1"},{"type":"function","title":"PropCheck.StateM.commands/2","doc":"Similar to `commands/1`, but generated command sequences always\nstart at a given state.\n\nIn this case, the first command is always\n`{:init, initial_state}` and is used to correctly initialize the state\nevery time the command sequence is run (i.e. during normal execution,\nwhile shrinking and when checking a counterexample). In this case,\n`mod:initial_state/0` is never called.","ref":"PropCheck.StateM.html#commands/2"},{"type":"type","title":"PropCheck.StateM.dynamic_state/0","doc":"A dynamic state can be anything and appears only during phase 2.","ref":"PropCheck.StateM.html#t:dynamic_state/0"},{"type":"type","title":"PropCheck.StateM.history/0","doc":"History of command execution in phase 2. It contains current dynamic state and\nthe result of the call.","ref":"PropCheck.StateM.html#t:history/0"},{"type":"callback","title":"PropCheck.StateM.initial_state/0","doc":"Specifies the symbolic initial state of the state machine.\n\nThis state\nwill be evaluated at command execution time to produce the actual initial\nstate. The function is not only called at command generation time, but\nalso in order to initialize the state every time the command sequence is\nrun (i.e. during normal execution, while shrinking and when checking a\ncounterexample). For this reason, it should be deterministic and\nself-contained.","ref":"PropCheck.StateM.html#c:initial_state/0"},{"type":"function","title":"PropCheck.StateM.more_commands/2","doc":"Increases the expected length of command sequences generated from\n`cmd_type` by a factor `n`.\n\n**CAVEAT** \nThis function does not work properly. My current guess is that this is\na limitation of how PropEr works with sizing an din particular resizing.\nThe commands list generator (`cmd_type`) is not a simple list which can\nbe sized easily, but a complex construct where the rather simple approach\nof resizing does not work as expected.","ref":"PropCheck.StateM.html#more_commands/2"},{"type":"callback","title":"PropCheck.StateM.next_state/3","doc":"Specifies the next state of the abstract state machine, given the\ncurrent state `s`, the symbolic `call` chosen and its result `res`. This\nfunction is called both at command generation and command execution time\nin order to update the model state, therefore the state `s` and the\nresult `Res` can be either symbolic or dynamic.","ref":"PropCheck.StateM.html#c:next_state/3"},{"type":"function","title":"PropCheck.StateM.parallel_commands/1","doc":"A special PropEr type which generates parallel test cases,\naccording to an abstract state machine specification.\n\nThe function takes as\ninput the name of a callback module, which contains the state machine\nspecification. The initial state is computed by `mod:initial_state/0`.","ref":"PropCheck.StateM.html#parallel_commands/1"},{"type":"function","title":"PropCheck.StateM.parallel_commands/2","doc":"Similar to `parallel_commands/1`, but generated command sequences\nalways start at a given state.","ref":"PropCheck.StateM.html#parallel_commands/2"},{"type":"type","title":"PropCheck.StateM.parallel_history/0","doc":"The history of concurrent execution of commands in phase 2.","ref":"PropCheck.StateM.html#t:parallel_history/0"},{"type":"type","title":"PropCheck.StateM.parallel_testcase/0","doc":"A parallel testcase consists of a sequential and a parallel component. The\nsequential component is a command sequence that is run first to put the system\nin a random state. The parallel component is a list containing 2 command\nsequences to be executed in parallel, each of them in a separate newly-spawned\nprocess.","ref":"PropCheck.StateM.html#t:parallel_testcase/0"},{"type":"callback","title":"PropCheck.StateM.postcondition/3","doc":"Specifies the postcondition that should hold about the result `res` of\nperforming `call`, given the dynamic state `s` of the abstract state\nmachine prior to command execution.\n\nThis function is called during\nruntime, this is why the state is dynamic.","ref":"PropCheck.StateM.html#c:postcondition/3"},{"type":"callback","title":"PropCheck.StateM.precondition/2","doc":"Specifies the precondition that should hold so that `call` can be\nincluded in the command sequence, given the current state `s` of the\nabstract state machine.\n\nIn case precondition doesn't hold, a new call is\nchosen using the `command/1` generator. If preconditions are very strict,\nit will take a lot of tries for PropEr to randomly choose a valid command.\nTesting will be stopped in case the `constraint_tries` limit is reached\n(see the `Options` section in the {@link proper} module documentation).\nPreconditions are also important for correct shrinking of failing\ntest cases. When shrinking command sequences, we try to eliminate commands\nthat do not contribute to failure, ensuring that all preconditions still\nhold. Validating preconditions is necessary because during shrinking we\nusually attempt to perform a call with the system being in a state\ndifferent from the state it was when initially running the test.","ref":"PropCheck.StateM.html#c:precondition/2"},{"type":"function","title":"PropCheck.StateM.print_report/3","doc":"Print pretty report of the failed command run.\n\nNote that it by default aliases every commands module, so only the last part\nof the namespace will be used.\n\nAccepts options:\n* `return_values` - whether to print return values after each command run\n  (default `true`),\n* `last_state` - whether section with the last state should be present\n  (default `true`),\n* `pre_cmd_state` - whether to print state prior to executed command\n  (default `false`),\n* `post_cmd_state` - whether to print state post executed command\n  (default `true`),\n* `cmd_args` - whether to print command arguments as literals\n  (default `true`),\n* `inspect_opts` - options passed to `inspect/2`\n* `alias` - set a list of modules and aliases, to be used to pretty print\ncommands. Accepts a `t:module/0`,  a tuple of type\n`{module(), alias :: module()}` or a list of them (default is to alias every\ncommand's module - if you want to disable default aliasing set it to `[]`).","ref":"PropCheck.StateM.html#print_report/3"},{"type":"type","title":"PropCheck.StateM.result/0","doc":"The outcome of the command sequence execution.","ref":"PropCheck.StateM.html#t:result/0"},{"type":"function","title":"PropCheck.StateM.run_commands/2","doc":"Evaluates a given symbolic command sequence `cmds` according to the\nstate machine specified in `mod`.\n\nThe result is a triple of the form\n`{history, dynamic_state, result}`, where:\n\n* `history` contains the execution history of all commands that were\n  executed without raising an exception. It contains tuples of the form\n  `{t:dynamic_state/0, t:term/0}`, specifying the state prior to\n  command execution and the actual result of the command.\n* `dynamicState` contains the state of the abstract state machine at\n  the moment when execution stopped. In case execution has stopped due to a\n  false postcondition, `dynamic_state` corresponds to the state prior to\n  execution of the last command.\n* `result` specifies the outcome of command execution. It can be\n  classified in one of the following categories:\n   \n    *ok*\n     All commands were successfully run and all postconditions were true.\n    *initialization error*\n     There was an error while evaluating the initial state.\n    *postcondition error*\n     A postcondition was false or raised an exception.\n    *precondition error*\n     A precondition was false or raised an exception.\n    *exception*\n     An exception was raised while running a command.","ref":"PropCheck.StateM.html#run_commands/2"},{"type":"function","title":"PropCheck.StateM.run_commands/3","doc":"Similar to `run_commands/2`, but also accepts an environment,\nused for symbolic variable evaluation during command execution. The\nenvironment consists of `{key::atom, value::any}` pairs. Keys may be\nused in symbolic variables (i.e. `{:var, key}`) within the command sequence\n`cmds`. These symbolic variables will be replaced by their corresponding\n`value` during command execution.","ref":"PropCheck.StateM.html#run_commands/3"},{"type":"function","title":"PropCheck.StateM.run_parallel_commands/2","doc":"Runs a given parallel test case according to the state machine\nspecified in `mod`.\n\nThe result is a triple of the form\n`{sequential_history, parallel_history, result}`, where:\n\n* `sequential_history` contains the execution history of the\n  sequential component.\n* `Parallel_history` contains the execution history of each of the\n  concurrent tasks.\n* `Result` specifies the outcome of the attempt to serialize command\n  execution, based on the results observed. It can be one of the following:\n     `ok`   `no_possible_interleaving`  \n\nSince the scheduler of the runtime system is quite predictable, you are advised to add\nmore turbulences in the execution traces by instrumenting the code under test\nvia `PropCheck.Instrument`.\n\nThe history shown when the execution fails, does not reveal in which exact interleaving of\ncommands the failure happens but simply shows the executed commands in both processes\nrunning concurrently. This is a feature of PropEr, which cannot easily changed.","ref":"PropCheck.StateM.html#run_parallel_commands/2"},{"type":"function","title":"PropCheck.StateM.run_parallel_commands/3","doc":"Similar to `run_parallel_commands/2`, but also accepts an\nenvironment used for symbolic variable evaluation, exactly as described in\n`run_commands/3`.","ref":"PropCheck.StateM.html#run_parallel_commands/3"},{"type":"function","title":"PropCheck.StateM.state_after/2","doc":"Returns the symbolic state after running a given command sequence,\naccording to the state machine specification found in `mod`.\n\nThe commands are not actually executed.","ref":"PropCheck.StateM.html#state_after/2"},{"type":"type","title":"PropCheck.StateM.symbolic_call/0","doc":"A symbolic call is the typical mfa-tuple plus the tag `:call`.","ref":"PropCheck.StateM.html#t:symbolic_call/0"},{"type":"type","title":"PropCheck.StateM.symbolic_state/0","doc":"A symbolic state can be anything and appears only during phase 1.","ref":"PropCheck.StateM.html#t:symbolic_state/0"},{"type":"type","title":"PropCheck.StateM.symbolic_var/0","doc":"Each result of a symbolic call is stored in a symbolic variable. Their values\nare opaque and can only used as whole.","ref":"PropCheck.StateM.html#t:symbolic_var/0"},{"type":"function","title":"PropCheck.StateM.zip/2","doc":"Behaves exactly like `Enum.zip/2`.\n\nZipping stops when the shortest list stops. This is\nuseful for zipping a command sequence with its (failing) execution history.","ref":"PropCheck.StateM.html#zip/2"},{"type":"behaviour","title":"PropCheck.StateM.DSL","doc":"_DEPRECATED_ : This module is deprecated, please use\n`PropCheck.StateM.ModelDSL` instead.\n\nThis module provides a shallow DSL (domain specific language) in Elixir\nfor property based testing of stateful systems.","ref":"PropCheck.StateM.DSL.html"},{"type":"behaviour","title":"The basic approach - PropCheck.StateM.DSL","doc":"Property based testing of stateful systems is different from ordinary property\nbased testing. Instead of testing operations and their effects on the\ndata structure directly, we construct a model of the system and generate a sequence\nof commands operating on both, the model and the system. Then we check that\nafter each command step, the system has evolved according to the model.\nThis is the same idea which is used in model checking and is sometimes called\na bisimulation.\n\nAfter defining a model, we have two phases during property execution.\nIn phase 1, the generators create a list of\n(symbolic) commands including their parameters to be run against the system under test\n(SUT). A state machine guides the generation of commands.\n\nIn phase 2, the commands are executed and the state machine checks that  the\nSUT is in the same state as the state machine. If an invalid state is\ndetected, then the command sequence is shrunk towards a shorter sequence\nserving then as counterexamples.\n\nThis approach works exactly the same as with `PropCheck.StateM` and\n`PropCheck.FSM`. The main difference is the API, grouping pre- and postconditions,\nstate transitions, and argument generators around the commands of the SUT. This\nleads towards a more logical locality compared to the former implementations.\nQuickCheck EQC has a similar approach for structuring their modern state machines.","ref":"PropCheck.StateM.DSL.html#module-the-basic-approach"},{"type":"behaviour","title":"The DSL - PropCheck.StateM.DSL","doc":"A state machine acting as a model of the SUT can be defined by focusing on\nstates or on transitions. We  focus here on the transitions. A transition is a\ncommand calling the SUT. Therefore the main phrase of the DSL is the `defcommand`\nmacro.\n\n    defcommand :find do\n      # define the rules for executing the find command here\n    end\n\nInside the `command` macro, we define all the rules which the command must\nobey. As an example, we discuss here as an example the slightly simplified\ncommand `:find` from `test/cache_dsl_test.exs`. The SUT is a cache\nimplementation based on an ETS and the model is is based on a list of\n(key/value)-pairs. This example is derived from [Fred Hebert's PropEr Testing,\nChapter 9](http://propertesting.com/book_stateful_properties.html)\n\nThe `find`-command is a call to the `find/1` API function. Its arguments are\ngenerated by `key()`, which boils down to numeric values. The arguments for\nthe command are defined by the function `args(state)` returning a list\nof generators. In our example, the arguments do not depend on the model state.\nNext, we need to define the execution of the command by defining function\n`impl/n`. This function takes as many arguments as  `args/1` has elements in\nthe argument list. The `impl`-function allows to apply conversion of\nparameters and return values to ease the testing. A typical example is the\nconversion of an `{:ok, value}` tuple to only `value` which can simplify\nworking with `value`.\n\n    defcommand :find do\n      def impl(key), do: Cache.find(key)\n      def args(_state), do: [key()]\n    end\n\nAfter defining how a command is executed, we need to define in which state\nthis is allowed. For this, we define function `pre/2`, taking the model state\nand the generated list of arguments to check whether this call is\nallowed in the current model state. In this particular example, `find` is always\nallowed, hence we return `true` without any further checking. This is also the\ndefault implementation and the reason why the precondition is missing\nin the test file.\n\n    defcommand :find do\n      def impl(key), do: Cache.find(key)\n      def args(_state), do: [key()]\n      def pre(_state, [_key]), do: true\n    end\n\nIf the precondition is satisfied, the call can happen. After the call, the SUT\ncan be in a different state and the model state must be updated according to\nthe mapping of the SUT to the model. The function `next/3` takes the state before\nthe call, the list of arguments and the symbolic or dynamic result (depending\non phase 1 or 2, respectively). `next/3` returns the  new model state.  Since\nsearching for a key in the cache does not modify the system nor the model\nstate, nothing has to be done. This is again the default implementation and thus\nleft out in the test file.\n\n    defcommand :find do\n      def impl(key), do: Cache.find(key)\n      def args(_state), do: [key()]\n      def pre(_state, [_key]), do: true\n      def next(old_state, _args, call_result), do: old_state\n    end\n\nThe missing part of the command definition is the post condition, checking\nthat after calling the system in phase 2 the system is in the expected state\ncompared the model. This check is implemented in function `post/3`, which\nagain has a trivial default implementation for post conditions that always returns\ntrue. In this example, we check if the `call_result` is `{:error, :not_found}`,\nthen we also do not find the key in our model list `entries`. The other case is\nthat if we a return value of `{:ok, val}`, then we also find the value via\nthe `key` in our list of `entries`.\n\n    defcommand :find do\n      def impl(key), do: Cache.find(key)\n      def args(_state), do: [key()]\n      def pre(_state, [_key]), do: true\n      def next(old_state, _args, _call_result), do: old_state\n      def post(entries, [key], call_result) do\n        case List.keyfind(entries, key, 0, false) do\n            false       -> call_result == {:error, :not_found}\n            {^key, val} -> call_result == {:ok, val}\n        end\n      end\n    end\n\nThis completes the DSL for command definitions.","ref":"PropCheck.StateM.DSL.html#module-the-dsl"},{"type":"behaviour","title":"Additional model elements - PropCheck.StateM.DSL","doc":"In addition to commands, we need to define the model itself. This is the\ningenious part of stateful property based testing! The initial state\nof the model must be implemented as the function `initial_state/0`. From this\nfunction, all model evolutions start. In our simplified cache example the\ninitial model is an empty list:\n\n    def initial_state(), do: []\n\nThe commands are generated with the same frequency by default. Often, this\nis not appropriate, e.g. in the cache example we expect many more `find` than\n`cache` commands. Therefore, commands can have a weight, which is technically used\ninside a `PropCheck.BasicTypes.frequency/1` generator. The weights are defined\nin callback function `c:weight/1`, taking the current model state and returning\na map of command and frequency pairs to be generated.  In our cache example\nwe want the `find` command to appear three times more often than other commands:\n\n    def weight(_state), do: %{find: 3, cache: 1, flush: 1}","ref":"PropCheck.StateM.DSL.html#module-additional-model-elements"},{"type":"behaviour","title":"The property to test - PropCheck.StateM.DSL","doc":"The property to test the stateful system is more or less the same for all systems.\nWe generate all commands via generator `commands/1`, which takes\na module with callbacks as parameter. Inside the test, we first start\nthe SUT, execute the commands with `run_commands/1`, stopping the SUT\nand evaluating the result of the executions as a boolean expression.\nThis boolean expression can be adorned with further functions and macros\nto analyze the generated commands (via `PropCheck.aggregate/2`) or to\ninspect the history if a failure occurs (via `PropCheck.when_fail/2`).\nIn the test cases, you find more examples of such adornments.\n\n    property \"run the sequential cache\", [:verbose] do\n      forall cmds <- commands(__MODULE__) do\n        Cache.start_link(@cache_size)\n        execution = run_commands(cmds)\n        Cache.stop()\n        (execution.result == :ok)\n      end\n    end","ref":"PropCheck.StateM.DSL.html#module-the-property-to-test"},{"type":"behaviour","title":"Increasing the Number of Commands in a Sequence - PropCheck.StateM.DSL","doc":"Sometimes issues can hide when the command sequences are short. In order to\ntease out these hidden bugs we can increase the number of commands generated\nby using the `max_size` option in our property.\n\n      property \"run the sequential cache\", [max_size: 250] do\n      forall cmds <- commands(__MODULE__) do\n        Cache.start_link(@cache_size)\n        execution = run_commands(cmds)\n        Cache.stop()\n        (execution.result == :ok)\n      end","ref":"PropCheck.StateM.DSL.html#module-increasing-the-number-of-commands-in-a-sequence"},{"type":"type","title":"PropCheck.StateM.DSL.command/0","doc":"A value of type `command` denotes the execution of a symbolic command and\nstoring its result in a symbolic variable.","ref":"PropCheck.StateM.DSL.html#t:command/0"},{"type":"type","title":"PropCheck.StateM.DSL.command_name/0","doc":"The name of a command must be an atom.","ref":"PropCheck.StateM.DSL.html#t:command_name/0"},{"type":"function","title":"PropCheck.StateM.DSL.command_names/1","doc":"Takes a list of generated commands and returns a list of\nmfa-tuples. This can be used for aggregation of commands.","ref":"PropCheck.StateM.DSL.html#command_names/1"},{"type":"function","title":"PropCheck.StateM.DSL.commands/1","doc":"Generates the command list for the given module","ref":"PropCheck.StateM.DSL.html#commands/1"},{"type":"macro","title":"PropCheck.StateM.DSL.defcommand/2","doc":"_DEPRECATED_ : This module is deprecated, please use\n`PropCheck.Statem.ModelDSL` instead.\n\nDefines a new command of the model.\n\nInside the command, local functions define\n* how the command is executed (`impl(...)`). This is required.\n* how the arguments in the current model state are generated (`args(state)`.\n  The default is the empty list of arguments.\n* if the command is allowed in the current model state (`pre(state, arg_list) :: boolean`)\n  This is `true` per default.\n* what the next state of the model is after the call (`next(old_state, arg_list, result) :: new_state`).\n  The default implementation does not change the model state, sufficient for\n  queries.\n* if the system under test is in the correct state after the call\n  (`post(old_state, arg_list, result) :: boolean`). This is `true` in the\n  default implementation.\n\nThese local functions inside the macro are effectively callbacks to guide and\nevolve the model state.","ref":"PropCheck.StateM.DSL.html#defcommand/2"},{"type":"type","title":"PropCheck.StateM.DSL.dynamic_state/0","doc":"A dynamic state can be anything and appears only during phase 2.","ref":"PropCheck.StateM.DSL.html#t:dynamic_state/0"},{"type":"type","title":"PropCheck.StateM.DSL.history_event/0","doc":"The history of command execution in phase 2 is stored in a history element.\nIt contains the current dynamic state and the call to be made.","ref":"PropCheck.StateM.DSL.html#t:history_event/0"},{"type":"callback","title":"PropCheck.StateM.DSL.initial_state/0","doc":"The initial state of the state machine is computed by this callback.","ref":"PropCheck.StateM.DSL.html#c:initial_state/0"},{"type":"type","title":"PropCheck.StateM.DSL.result_t/0","doc":"The result of the command execution. It contains either the state of the failing\nprecondition, the command's return value of the failing postcondition,\nthe exception values or `:ok` if everything is fine.","ref":"PropCheck.StateM.DSL.html#t:result_t/0"},{"type":"function","title":"PropCheck.StateM.DSL.run_commands/1","doc":"Runs the list of generated commands according to the model.\n\nReturns the result, the history and the final state of the model.\n\nDue to an internal refactoring and to achieve a common API with the `PropCheck.StateM`\nmodule, we changed the API for `run_commands`. This implementation infers the\ncallback module from the first generated command. Usually, this will be the case,\nbut we cannot rely on that.","ref":"PropCheck.StateM.DSL.html#run_commands/1"},{"type":"function","title":"PropCheck.StateM.DSL.run_commands/2","doc":"Runs the list of generated commands according to the model.\n\nReturns the result, the history and the final state of the model.","ref":"PropCheck.StateM.DSL.html#run_commands/2"},{"type":"type","title":"PropCheck.StateM.DSL.state_call/0","doc":"The sequence of calls consists of state and symbolic calls.","ref":"PropCheck.StateM.DSL.html#t:state_call/0"},{"type":"type","title":"PropCheck.StateM.DSL.state_t/0","doc":"The combination of symbolic and dynamic states are required for functions\nwhich are used in both phases 1 and 2.","ref":"PropCheck.StateM.DSL.html#t:state_t/0"},{"type":"type","title":"PropCheck.StateM.DSL.symbolic_call/0","doc":"A symbolic call is the typical mfa-tuple plus the indicator `:call`.","ref":"PropCheck.StateM.DSL.html#t:symbolic_call/0"},{"type":"type","title":"PropCheck.StateM.DSL.symbolic_state/0","doc":"A symbolic state can be anything and appears only during phase 1.","ref":"PropCheck.StateM.DSL.html#t:symbolic_state/0"},{"type":"type","title":"PropCheck.StateM.DSL.symbolic_var/0","doc":"Each result of a symbolic call is stored in a symbolic variable. Their values\nare opaque and can only used as whole.","ref":"PropCheck.StateM.DSL.html#t:symbolic_var/0"},{"type":"type","title":"PropCheck.StateM.DSL.t/0","doc":"The combined result of the test. It contains the history of all executed commands,\nthe final state, the final result and the environment, mapping symbolic\nvars to their actual values. Everything is fine, if `result` is `:ok`.","ref":"PropCheck.StateM.DSL.html#t:t/0"},{"type":"callback","title":"PropCheck.StateM.DSL.weight/1","doc":"_DEPRECATED_ : This module is deprecated, please use\n`PropCheck.Statem.ModelDSL` instead.\n\nThe optional weights for the command generation. It takes the current\nmodel state and returns a map of command/weight pairs. Commands,\nwhich are not allowed in a specific state, should be omitted, since\na frequency of `0` is not allowed.\n\n    def weight(state), do: %{x: 1, y: 1, a: 2, b: 2}","ref":"PropCheck.StateM.DSL.html#c:weight/1"},{"type":"behaviour","title":"PropCheck.StateM.ModelDSL","doc":"This module provides a shallow DSL (domain specific language) in Elixir for\nproperty based testing of stateful systems. It's built upon `PropCheck.StateM`\nand all it's the characteristics apply here as well. It's a replacement for\n`PropCheck.StateM.DSL`.","ref":"PropCheck.StateM.ModelDSL.html"},{"type":"behaviour","title":"The basic approach - PropCheck.StateM.ModelDSL","doc":"Property based testing of stateful systems is different from ordinary property\nbased testing. Instead of testing operations and their effects on the data\nstructure directly, we construct a model of the system and generate a sequence\nof commands operating on both, the model and the system. Then we check that\nafter each command step, the system has evolved accordingly to the model.\nThis is the same idea which is used in model checking and is sometimes called\na bisimulation.\n\nAfter defining a model, we have two phases during executing the property.  In\nphase 1, the generators create a list of (symbolic) commands including their\nparameters to be run against the system under test (SUT). A state machine\nguides the generation of commands.\n\nIn phase 2, the commands are executed and the state machine checks that  the\nSUT is in the same state as the state machine. If an invalid state is\ndetected, then the command sequence is shrunk towards a shorter sequence\nserving then as counterexamples.\n\nThis approach works exactly the same as with `PropCheck.StateM` and\n`PropCheck.FSM`. The main difference is the API, grouping pre- and\npostconditions, state transitions around the commands of the SUT. This leads\ntowards more logical locality compared to the former implementations.\nQuickCheck EQC has a similar approach for structuring their modern state\nmachines.","ref":"PropCheck.StateM.ModelDSL.html#module-the-basic-approach"},{"type":"behaviour","title":"The DSL - PropCheck.StateM.ModelDSL","doc":"A state machine acting as a model of the SUT can be defined by focusing on\nstates or on transitions. We focus here on the transitions. A transition is a\ncommand calling the SUT. Therefore the main phrase of the DSL is the\n`defcommand` macro.\n\n    defcommand :find do\n      # define the rules for executing the find command here\n    end\n\nInside the `defcommand` macro, we define all the rules which the command must\nobey. As an example, we discuss here as an example the slightly simplified\ncommand `:find` from `test/cache_dsl_test.exs`. The SUT is a cache\nimplementation based on an ETS and the model is is based on a list of\n(key/value)-pairs. This example is derived from [Fred Hebert's PropEr Testing,\nChapter 9](http://propertesting.com/book_stateful_properties.html)\n\nThe `find`-command is a call to the `find/1` API function. Its arguments are\ngenerated in `c:command_gen/1` (described later) callback, which for this\ncommand is using just one argument, a `key()` generator. Next, we need to\ndefine the execution of the command by defining function `impl/n`.  The\n`impl`-function allows to apply conversion of parameters and return values to\nease the testing. A typical example is the conversion of an `{:ok, value}`\ntuple to only `value` which can simplify working with `value`.\n\n    defcommand :find do\n      def impl(key), do: Cache.find(key)\n    end\n\nAfter defining how a command is executed, we need to define in which state\nthis is allowed. For this, we define function `pre/2`, taking the model state\nand the generated list of arguments to check whether this call is allowed in\nthe current model state. In this particular example, `find` is always allowed,\nhence we return `true` without any further checking. This is also the default\nimplementation and the reason why the precondition is missing in the test\nfile.\n\n    defcommand :find do\n      def impl(key), do: Cache.find(key)\n      def pre(_state, [_key]), do: true\n    end\n\nIf the precondition is satisfied, the call can happen. After the call, the SUT\ncan be in a different state and the model state must be updated according to\nthe mapping of the SUT to the model. The function `next/3` takes the state\nbefore the call, the list of arguments and the symbolic or dynamic result\n(depending on phase 1 or 2, respectively). `next/3` returns the new model\nstate. Since searching for a key in the cache does not modify the system nor\nthe model state, nothing has to be done. This is again the default\nimplementation and thus left out in the test file.\n\n    defcommand :find do\n      def impl(key), do: Cache.find(key)\n      def pre(_state, [_key]), do: true\n      def next(old_state, _args, call_result), do: old_state\n    end\n\nThe missing part of the command definition is the post condition, checking\nthat after calling the system in phase 2, the system is in the expected state\ncompared the model. This check is implemented in function `post/3`, which\nagain has a trivial default implementation for post conditions that always\nreturns true. In this example, we check if we can find the key in our list of\n`entries` and if we do, we check if `call_result` resulted in `{:ok, val}`. Or\nif we don't found it, we check if the SUT also cannot find it by comparing if\n`call_result` returned `{:error, :not_found}`.\n\n    defcommand :find do\n      def impl(key), do: Cache.find(key)\n      def pre(_state, [_key]), do: true\n      def next(old_state, _args, _call_result), do: old_state\n      def post(entries, [key], call_result) do\n        case List.keyfind(entries, key, 0, false) do\n            false       -> call_result == {:error, :not_found}\n            {^key, val} -> call_result == {:ok, val}\n        end\n      end\n    end\n\nThis completes the DSL for command definitions.","ref":"PropCheck.StateM.ModelDSL.html#module-the-dsl"},{"type":"behaviour","title":"Additional model elements - PropCheck.StateM.ModelDSL","doc":"In addition to commands, we need to define the model itself. This is the\ningenious part of stateful property based testing! The initial state of the\nmodel must be implemented as the function `initial_state/0`. It doesn't accept\nany arguments, because this function has to be deterministic. From this\nfunction, all model evolutions start. In our simplified cache example the\ninitial model is an empty list:\n\n    def initial_state(), do: []\n\nThe sequence of commands to be run is generated repeatedly by\n`c:command_gen/1` callback. The generator has to return a tuple of with a\ncommand name and a list of it's arguments (a list of generators). This\ncallback expects the current state as an argument, which often is used to\ndetermine the next one from a set of appropriate commands (e.g. there might\nnot be much sense in calling the `delete_user` command, if there are no users\nin the system yet). Usually a `PropCheck.BasicTypes.oneof/1` or\n`PropCheck.BasicTypes.frequency/1` generators are used to pick one of possible\ncommands. In our cache example we want the `find` command to appear three\ntimes more often than other commands:\n\n    def command_gen(_state) do\n      frequency([\n        {3, {:find, [key()]}},\n        {1, {:cache, [key(), val()]}},\n        {1, {:flush, []}}\n      ])","ref":"PropCheck.StateM.ModelDSL.html#module-additional-model-elements"},{"type":"behaviour","title":"The property to test - PropCheck.StateM.ModelDSL","doc":"The property to test the stateful system is more or less the same for all systems.\nWe generate all commands via generator `commands/1`, which takes\na module with callbacks as parameter. Inside the test, we first start\nthe SUT, execute the commands with `run_commands/2`, stopping the SUT\nand evaluating the result of the executions as a boolean expression.\nThis boolean expression can be adorned with further functions and macros\nto analyze the generated commands (via `PropCheck.aggregate/2`) or to\ninspect the history if a failure occurs (via `PropCheck.when_fail/2`).\nIn the test cases, you find more examples of such adornments.\n\n    property \"run the sequential cache\", [:verbose] do\n      forall cmds <- commands(__MODULE__) do\n        Cache.start_link(@cache_size)\n        {_history, _state, result} = run_commands(__MODULE__, cmds)\n        Cache.stop()\n        (result == :ok)\n      end\n    end","ref":"PropCheck.StateM.ModelDSL.html#module-the-property-to-test"},{"type":"behaviour","title":"Increasing the Number of Commands in a Sequence - PropCheck.StateM.ModelDSL","doc":"Sometimes issues can hide when the command sequences are short. In order to\ntease out these hidden bugs we can increase the number of commands generated\nby using the `max_size` option in our property.\n\n      property \"run the sequential cache\", [max_size: 250] do\n      forall cmds <- commands(__MODULE__) do\n        Cache.start_link(@cache_size)\n        {_history, _state, result} = run_commands(__MODULE__, cmds)\n        Cache.stop()\n        (result == :ok)\n      end","ref":"PropCheck.StateM.ModelDSL.html#module-increasing-the-number-of-commands-in-a-sequence"},{"type":"type","title":"PropCheck.StateM.ModelDSL.command/0","doc":"A value of type `command` denotes the execution of a symbolic command and\nstoring its result in a symbolic variable.","ref":"PropCheck.StateM.ModelDSL.html#t:command/0"},{"type":"callback","title":"PropCheck.StateM.ModelDSL.command_gen/1","doc":"Generates a symbolic call to be included in the command sequence, given the\ncurrent state `s` of the abstract state machine. Must return a type that\ngenerates tuples\n`{command_name :: atom, args :: [PropCheck.BasicTypes.type]}`.\n\nHowever, before the call is actually included, a precondition is checked. This\nfunction will be repeatedly called to produce the next call to be included in\nthe test case.","ref":"PropCheck.StateM.ModelDSL.html#c:command_gen/1"},{"type":"type","title":"PropCheck.StateM.ModelDSL.command_list/0","doc":"A sequence of commands.","ref":"PropCheck.StateM.ModelDSL.html#t:command_list/0"},{"type":"function","title":"PropCheck.StateM.ModelDSL.command_names/1","doc":"Extracts the names of the commands from a given command sequence, in\nthe form of MFAs.\n\nIt is useful in combination with functions such as\n`PropCheck.aggregate/2` in order to collect statistics about command\nexecution.","ref":"PropCheck.StateM.ModelDSL.html#command_names/1"},{"type":"function","title":"PropCheck.StateM.ModelDSL.commands/1","doc":"A special PropEr type which generates random command sequences,\naccording to an abstract state machine specification.\n\nThe function takes as\ninput the name of a callback module, which contains the state machine\nspecification. The initial state is computed by `mod:initial_state/0`.","ref":"PropCheck.StateM.ModelDSL.html#commands/1"},{"type":"function","title":"PropCheck.StateM.ModelDSL.commands/2","doc":"Similar to `commands/1`, but generated command sequences always\nstart at a given state.\n\nIn this case, the first command is always\n`{:init, initial_state}` and is used to correctly initialize the state\nevery time the command sequence is run (i.e. during normal execution,\nwhile shrinking and when checking a counterexample). In this case,\n`mod:initial_state/0` is never called.","ref":"PropCheck.StateM.ModelDSL.html#commands/2"},{"type":"function","title":"PropCheck.StateM.ModelDSL.def_commands/0","doc":"","ref":"PropCheck.StateM.ModelDSL.html#def_commands/0"},{"type":"function","title":"PropCheck.StateM.ModelDSL.def_next_states/1","doc":"","ref":"PropCheck.StateM.ModelDSL.html#def_next_states/1"},{"type":"function","title":"PropCheck.StateM.ModelDSL.def_postconds/1","doc":"","ref":"PropCheck.StateM.ModelDSL.html#def_postconds/1"},{"type":"function","title":"PropCheck.StateM.ModelDSL.def_preconds/1","doc":"","ref":"PropCheck.StateM.ModelDSL.html#def_preconds/1"},{"type":"macro","title":"PropCheck.StateM.ModelDSL.defcommand/2","doc":"Defines a new command of the model.\n\nInside the command, local functions define\n* how the command is executed: `impl(...)` - this is required,\n* if the command is allowed in the current model state:\n  `pre(state, arg_list) :: boolean` - this is `true` per default,\n* what the next state of the model is after the call:\n  `next(old_state, arg_list, result) :: new_state` - the default\n  implementation does not change the model state, sufficient for queries,\n* if the system under test is in the correct state after the call:\n  `post(old_state, arg_list, result) :: boolean` - this is `true` in the\n  default implementation.\n\nThese local functions inside the macro are effectively callbacks to guide and\nevolve the model state.","ref":"PropCheck.StateM.ModelDSL.html#defcommand/2"},{"type":"type","title":"PropCheck.StateM.ModelDSL.dynamic_state/0","doc":"A dynamic state can be anything and appears only during phase 2.","ref":"PropCheck.StateM.ModelDSL.html#t:dynamic_state/0"},{"type":"type","title":"PropCheck.StateM.ModelDSL.history/0","doc":"History of command execution in phase 2. It contains current dynamic state and\nthe result of the call.","ref":"PropCheck.StateM.ModelDSL.html#t:history/0"},{"type":"callback","title":"PropCheck.StateM.ModelDSL.initial_state/0","doc":"Specifies the symbolic initial state of the state machine.\n\nThis state will be evaluated at command execution time to produce the actual\ninitial state. The function is not only called at command generation time, but\nalso in order to initialize the state every time the command sequence is run\n(i.e. during normal execution, while shrinking and when checking a\ncounterexample). For this reason, it should be deterministic and\nself-contained.","ref":"PropCheck.StateM.ModelDSL.html#c:initial_state/0"},{"type":"function","title":"PropCheck.StateM.ModelDSL.more_commands/2","doc":"Increases the expected length of command sequences generated from\n`cmd_type` by a factor `n`.\n\n**CAVEAT** \nThis function does not work properly. My current guess is that this is\na limitation of how PropEr works with sizing an din particular resizing.\nThe commands list generator (`cmd_type`) is not a simple list which can\nbe sized easily, but a complex construct where the rather simple approach\nof resizing does not work as expected.","ref":"PropCheck.StateM.ModelDSL.html#more_commands/2"},{"type":"function","title":"PropCheck.StateM.ModelDSL.parallel_commands/1","doc":"A special PropEr type which generates parallel test cases,\naccording to an abstract state machine specification.\n\nThe function takes as\ninput the name of a callback module, which contains the state machine\nspecification. The initial state is computed by `mod:initial_state/0`.","ref":"PropCheck.StateM.ModelDSL.html#parallel_commands/1"},{"type":"function","title":"PropCheck.StateM.ModelDSL.parallel_commands/2","doc":"Similar to `parallel_commands/1`, but generated command sequences\nalways start at a given state.","ref":"PropCheck.StateM.ModelDSL.html#parallel_commands/2"},{"type":"type","title":"PropCheck.StateM.ModelDSL.parallel_history/0","doc":"The history of concurrent execution of commands in phase 2.","ref":"PropCheck.StateM.ModelDSL.html#t:parallel_history/0"},{"type":"type","title":"PropCheck.StateM.ModelDSL.parallel_testcase/0","doc":"A parallel testcase consists of a sequential and a parallel component. The\nsequential component is a command sequence that is run first to put the system\nin a random state. The parallel component is a list containing 2 command\nsequences to be executed in parallel, each of them in a separate newly-spawned\nprocess.","ref":"PropCheck.StateM.ModelDSL.html#t:parallel_testcase/0"},{"type":"function","title":"PropCheck.StateM.ModelDSL.print_report/3","doc":"Print pretty report of the failed command run.\n\nNote that it by default aliases every commands module, so only the last part\nof the namespace will be used.\n\nAccepts options:\n* `return_values` - whether to print return values after each command run\n(default `true`),\n* `last_state` - whether section with the last state should be present\n(default `true`),\n* `pre_cmd_state` - whether to print state prior to executed command\n(default `false`),\n* `post_cmd_state` - whether to print state post executed command\n(default `true`),\n* `cmd_args` - whether to print command arguments as literals\n(default `true`),\n* `inspect_opts` - options passed to `inspect/2`\n* `alias` - set a list of modules and aliases, to be used to pretty print\ncommands. Accepts a `t:module/0`,  a tuple of type\n`{module(), alias :: module()}` or a list of them (default is to alias every\ncommand's module - if you want to disable default aliasing set it to `[]`).","ref":"PropCheck.StateM.ModelDSL.html#print_report/3"},{"type":"type","title":"PropCheck.StateM.ModelDSL.result/0","doc":"The outcome of the command sequence execution.","ref":"PropCheck.StateM.ModelDSL.html#t:result/0"},{"type":"function","title":"PropCheck.StateM.ModelDSL.run_commands/2","doc":"Evaluates a given symbolic command sequence `cmds` according to the\nstate machine specified in `mod`.\n\nThe result is a triple of the form\n`{history, dynamic_state, result}`, where:\n\n* `history` contains the execution history of all commands that were\n  executed without raising an exception. It contains tuples of the form\n  `{t:dynamic_state, t:term}`, specifying the state prior to\n  command execution and the actual result of the command.\n* `dynamicState` contains the state of the abstract state machine at\n  the moment when execution stopped. In case execution has stopped due to a\n  false postcondition, `dynamic_state` corresponds to the state prior to\n  execution of the last command.\n* `result` specifies the outcome of command execution. It can be\n  classified in one of the following categories:\n   \n    *ok*\n     All commands were successfully run and all postconditions were true.\n    *initialization error*\n     There was an error while evaluating the initial state.\n    *postcondition error*\n     A postcondition was false or raised an exception.\n    *precondition error*\n     A precondition was false or raised an exception.\n    *exception*\n     An exception was raised while running a command.","ref":"PropCheck.StateM.ModelDSL.html#run_commands/2"},{"type":"function","title":"PropCheck.StateM.ModelDSL.run_commands/3","doc":"Similar to `run_commands/2`, but also accepts an environment,\nused for symbolic variable evaluation during command execution. The\nenvironment consists of `{key::atom, value::any}` pairs. Keys may be\nused in symbolic variables (i.e. `{:var, key}`) within the command sequence\n`cmds`. These symbolic variables will be replaced by their corresponding\n`value` during command execution.","ref":"PropCheck.StateM.ModelDSL.html#run_commands/3"},{"type":"function","title":"PropCheck.StateM.ModelDSL.run_parallel_commands/2","doc":"Runs a given parallel test case according to the state machine\nspecified in `mod`.\n\nThe result is a triple of the form\n`{sequential_history, parallel_history, result}`, where:\n\n* `sequential_history` contains the execution history of the\n  sequential component.\n* `Parallel_history` contains the execution history of each of the\n  concurrent tasks.\n* `Result` specifies the outcome of the attempt to serialize command\n  execution, based on the results observed. It can be one of the following:\n     `ok`   `no_possible_interleaving`  \n\nSince the scheduler of the runtime system is quite predictable, you are advised to add\nmore turbulences in the execution traces by instrumenting the code under test\nvia `PropCheck.Instrument`.\n\nThe history shown when the execution fails, does not reveal in which exact interleaving of\ncommands the failure happens but simply shows the executed commands in both processes\nrunning concurrently. This is a feature of PropEr, which cannot easily changed.","ref":"PropCheck.StateM.ModelDSL.html#run_parallel_commands/2"},{"type":"function","title":"PropCheck.StateM.ModelDSL.run_parallel_commands/3","doc":"Similar to `run_parallel_commands/2`, but also accepts an\nenvironment used for symbolic variable evaluation, exactly as described in\n`run_commands/3`.","ref":"PropCheck.StateM.ModelDSL.html#run_parallel_commands/3"},{"type":"function","title":"PropCheck.StateM.ModelDSL.state_after/2","doc":"Returns the symbolic state after running a given command sequence,\naccording to the state machine specification found in `mod`.\n\nThe commands are not actually executed.","ref":"PropCheck.StateM.ModelDSL.html#state_after/2"},{"type":"type","title":"PropCheck.StateM.ModelDSL.symbolic_call/0","doc":"A symbolic call is the typical mfa-tuple plus the tag `:call`.","ref":"PropCheck.StateM.ModelDSL.html#t:symbolic_call/0"},{"type":"type","title":"PropCheck.StateM.ModelDSL.symbolic_state/0","doc":"A symbolic state can be anything and appears only during phase 1.","ref":"PropCheck.StateM.ModelDSL.html#t:symbolic_state/0"},{"type":"type","title":"PropCheck.StateM.ModelDSL.symbolic_var/0","doc":"Each result of a symbolic call is stored in a symbolic variable. Their values\nare opaque and can only used as whole.","ref":"PropCheck.StateM.ModelDSL.html#t:symbolic_var/0"},{"type":"function","title":"PropCheck.StateM.ModelDSL.zip/2","doc":"Behaves exactly like `Enum.zip/2`.\n\nZipping stops when the shortest list stops. This is\nuseful for zipping a command sequence with its (failing) execution history.","ref":"PropCheck.StateM.ModelDSL.html#zip/2"},{"type":"module","title":"PropCheck.TargetedPBT","doc":"This module defines the top-level behaviour for targeted property-based testing (TPBT).\nUsing TPBT the input generation is no longer random, but guided by a search strategy to\nincrease the probability of finding failing input. For this to work the user has to specify\na search strategy and also needs to extract utility-values from the system under test that\nthe search strategy then tries to maximize.\n\nTo use TPBT the test specification macros `forall_targeted`, `exists`, and `not_exists` are used.\nThe typical structure for a targeted property looks as follows:\n\n    property prop_target do          # Try to check that\n      exists input <- params do      # some input exists that fulfills the property.\n          uv = sut.run(input)        # Do so by running SUT with input\n          maximize(uv)               # and maximize its utility value\n          uv < threshold             # up to some threshold, the property condition.\n      end\n    end","ref":"PropCheck.TargetedPBT.html"},{"type":"module","title":"Some thoughts on strategies how to use targeted properties - PropCheck.TargetedPBT","doc":"Targeted PBT is a rather new technology and really fascinating. But when should you to use it and when are\nthe classical technologies more suitable? Here are some thoughts on that topic based on the\nlimited experience we currently have.\n\nIf you want to test interesting parts of your algorithms or data structures, you would typically invest\ninto more elaborated generators. If you are working on binary trees and you need more lefty trees, then a new\ngenerator is required that somehow generates trees of that particular shape. The big advantage here is that\nyou can use all of the standard functions and macros, in particular for collecting statistics about the quality\nof the generated data (e.g. by `PropCheck.collect/2` and its friends).\n\nAnother approach would be to use targeted properties\nas shown in `targeted_tree_test.exs`: You use a rather simple data generator and define a measuring function\non the data to express the \"leftiness\" of the tree. Equiped with that, a target property searches automatically\nfor data the maximizes (or minimizes) the measuring function (often called utility value or function). Not\ninventing a clever data generators comes with a price - there is no free lunch...\n* You need some _relevant_ property of your generated data which you can measure, i.e. you need a function\n  from your data to the real numbers. This is not always possible.\n* Searching for an optimum takes more time than simply generating random data: the run-time of the properties\n  increases.\n* Data collecting functions and macros such as `PropCheck.collect/2` are currently not available. This\n  it the reason why in `targeted_tree_test.exs` we use print-outs to show and verify the generated data.\n* Counter examples and shrinking are not available\n* The current implementation in PropEr does not work well together with recursive data generators, which\n  renders the approach unusable for state-based PBT.\n\nBut, of course, you gain also something. You can use rather straight data generators and let the searching\nalgorithm find the interesting parts with respect to the measuring function.\n\nIn `level_tpbt_test.exs` a very different approach is used. Here the basic idea is to verify that a data\nstructure (here: a maze in a computer game) has a proper structure (here: there exists at least one valid\npath from the entrance to the exit of the maze). The function to minimize is the distance from the end of\nthe path to the exit position. The searching algorithm then optimizes the path length for a minimal\ndistance until the exit is found. For more complex mazes, it is required to adopt the amount of `search_steps`\nand the `neighbor_hood` function to find the exit. They takes over the role `numtests` and `resize` to\nenlarge or refit the generated data for the next search step.\n\nYou can combine approaches by using a classical generator for e.g. generating a new maze, and then use\ninside a targeted property to find a path to the maze's exit. This would be roughly like this:\n\n    forall maze <- maze_generator() do\n      exists p <- path_generator() do\n        pos = Maze.follow_path(maze, maze.entry_pos, p)\n        uv = distance(pos, maze.exit_pos)\n        minimize(uv)\n        pos == maze.exit_pos\n      end\n    end","ref":"PropCheck.TargetedPBT.html#module-some-thoughts-on-strategies-how-to-use-targeted-properties"},{"type":"module","title":"How the targeted properties relate - PropCheck.TargetedPBT","doc":"The targeted macros `forall_targeted`, `exists` and `not_exists` are related to each other.\nThe universal laws of quantors from first-order logic apply here as well (cf.\n[provable entities in first-order logic](https://en.wikipedia.org/wiki/First-order_logic#Provable_identities))\nand explain why some conditions in the test examples are constructed the way they are.\n\nIn the following, we use the variable `x`, the generator `x_gen()` and a boolean predicate `p()`. The\nterm `<==>` means that the expression on both sides are equivalent.\n\n    forall_targeted x <- x_gen(), do: p(x)\n       <==> not_exists x <- x_gen(), do: not(p(x))\n\n    exists x <- x_gen(), do: p(x)\n       <==> forall_targeted x <- x_gen(), do: not(p(x))\n            |> fails()\n\n    not_exists x <- x_gen(), do: p(x)\n       <==> forall_targeted x <- x_gen(), do: not(p(x))","ref":"PropCheck.TargetedPBT.html#module-how-the-targeted-properties-relate"},{"type":"module","title":"Options - PropCheck.TargetedPBT","doc":"For targeted properties exists a new option:\n* `{:search_steps, non_negative_number}`  \n  takes an integer defining how many search steps the searching algorithm takes.\n  Its default value is `1_000`.  The effect of `search_steps` is similar to `num_tests` for\n  ordinary properties. `num_tests` has no effect on the search strategy. This helps when you\n  combine a regular property with search strategy, e.g. generating a configuration parameter\n  and search for specific properties to hold depending on that parameter.\n\nSome of the documentation is taken directly from PropEr.","ref":"PropCheck.TargetedPBT.html#module-options"},{"type":"macro","title":"PropCheck.TargetedPBT.exists/2","doc":"The `exists` macro uses the targeted PBT component of PropEr to try\nto find one instance of `xs` that makes the `prop` return `true`.\n\nIf such a `xs`\nis found, the property passes. Note that there is no counterexample if no\nsuch `xs` could be found.","ref":"PropCheck.TargetedPBT.html#exists/2"},{"type":"macro","title":"PropCheck.TargetedPBT.forall_targeted/2","doc":"The `forall_targeted` macros uses the targeted PBT component of PropEr to try\nthat all instances of `xs` fulfill property `prop`.\n\nIn contrast to `exists`, often the property here is negated.","ref":"PropCheck.TargetedPBT.html#forall_targeted/2"},{"type":"macro","title":"PropCheck.TargetedPBT.maximize/1","doc":"This macro tells the search strategy to maximize the value `fitness`.","ref":"PropCheck.TargetedPBT.html#maximize/1"},{"type":"macro","title":"PropCheck.TargetedPBT.minimize/1","doc":"This macro tells the search strategy to minimize the value `fitness` and\nis equivalent to `maximize(-fitness)`.","ref":"PropCheck.TargetedPBT.html#minimize/1"},{"type":"macro","title":"PropCheck.TargetedPBT.not_exists/2","doc":"The `not_exists` macro uses the targeted PBT component of PropEr to try\nto find one instance of `xs` that makes the `prop` return `false`.\n\nIf such a `xs`\nis found the property passes. Note that there is no counterexample if no\nsuch `xs` could be found.","ref":"PropCheck.TargetedPBT.html#not_exists/2"},{"type":"macro","title":"PropCheck.TargetedPBT.user_matcher/2","doc":"This overwrites the structural matching of PropEr for the generator with the user provided\n`matcher` function.\n\nThe `matcher` should be of type `:proper_gen_next.matcher()`","ref":"PropCheck.TargetedPBT.html#user_matcher/2"},{"type":"macro","title":"PropCheck.TargetedPBT.user_nf/2","doc":"This uses the neighborhood function `nf` instead of PropEr's\nconstructed neighborhood function for this generator.\n\nThe neighborhood function `nf` should be of type\n`fun(any(), {Depth :: number(), Temperature::float()} -> any()`","ref":"PropCheck.TargetedPBT.html#user_nf/2"},{"type":"module","title":"PropCheck.YieldInstrumenter","doc":"Instruments with prepending `:erlang.yield/0` for calls typical concurrency bug\naware functions.","ref":"PropCheck.YieldInstrumenter.html"},{"type":"extras","title":"PropCheck - Property based testing for Elixir","doc":"# PropCheck - Property based testing for Elixir\n\n`PropCheck` is a testing library, that provides a wrapper around PropEr, an Erlang\nbased property testing framework in the spirit of QuickCheck.\n\n![Elixir CI](https://github.com/alfert/propcheck/workflows/Elixir%20CI/badge.svg)\n[![Module Version](https://img.shields.io/hexpm/v/propcheck.svg)](https://hex.pm/packages/propcheck)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/propcheck/)\n[![Gitter](https://badges.gitter.im/propcheck/community.svg)](https://gitter.im/propcheck/community?utm_source=badge&utm_medium=badge&utm_campaign=pr-badge)\n[![Total Download](https://img.shields.io/hexpm/dt/propcheck.svg)](https://hex.pm/packages/propcheck)\n[![License](https://img.shields.io/hexpm/l/propcheck.svg)](https://hex.pm/packages/propcheck)\n[![Last Updated](https://img.shields.io/github/last-commit/alfert/propcheck.svg)](https://github.com/alfert/propcheck/commits/master)","ref":"readme.html"},{"type":"extras","title":"Installation - PropCheck - Property based testing for Elixir","doc":"To use PropCheck with your project, add it as a dependency to `mix.exs`:\n\n```elixir\ndefp deps do\n  [\n    {:propcheck, \"~> 1.4\", only: [:test, :dev]}\n  ]\nend\n```\n\nFrom PropCheck 1.3.0 onwards, we require at least Elixir 1.7 since in Elixir 1.11 function `get_stracktrace()` is deprecated.","ref":"readme.html#installation"},{"type":"extras","title":"Changes - PropCheck - Property based testing for Elixir","doc":"Relevant changes are documented in the [Changelog](changelog.html), on [GitHub\nfollow this link](CHANGELOG.md).","ref":"readme.html#changes"},{"type":"extras","title":"Basic Usage and Build Configuration - PropCheck - Property based testing for Elixir","doc":"PropCheck allows to define properties, which are automatically executed via `ExUnit`\nwhen running `mix test`. You find relevant information here:\n\n* Details about the `property` macro are found in `PropCheck.Properties`,\n* Details about how to specify the property conditions are documented in\n  `PropCheck`,\n* The basic data generators are found in `PropCheck.BasicTypes`,\n* For property testing of state-based systems, take a look at\n  `PropCheck.StateM.ModelDSL` for the new DSL (since PropCheck 1.1.0-rc1),\n  which is a layer on top of `PropCheck.StateM`.\n* The new targeted property based testing approach (TBPT) employing an automated\n  search strategy towards more interesting test data is described in\n  `PropCheck.TargetedPBT`.\n\nFor PropCheck, there is only one configuration option. All counter examples found\nare stored in a file, the name of which is configurable in `mix.exs` as part of\nthe `project` configuration:\n\n```elixir\ndef project() do\n  [ # many other options\n    propcheck: [counter_examples: \"filename\"]\n  ]\nend\n```\n\nNote that the path can also be set as part of the application environment:\n\n```\nconfig :propcheck, counter_examples: \"filename\"\n```\n\nIf both the project configuration and the application environment are present, the\napplication environment is chosen.\n\nPer default, the counter examples file is stored in the build directory (`_build`),\nindependent from the build environment, in the file `propcheck.ctex`. The file can\nbe removed using `mix propcheck.clean`. Note that this task is only available if PropCheck\nis also available in `:dev`. Otherwise, `MIX_ENV=test mix propcheck.clean` can be used.","ref":"readme.html#basic-usage-and-build-configuration"},{"type":"extras","title":"Setting PropCheck into Verbose Mode - PropCheck - Property based testing for Elixir","doc":"Properties in PropCheck can be run in quiet or verbose mode. Usually, quiet is the default. To\nenable verbose mode without requiring to touch the source code, the environment variable `PROPCHECK_VERBOSE`\ncan be used. If this is set to 1, the `forall` macro prints verbose output.","ref":"readme.html#setting-propcheck-into-verbose-mode"},{"type":"extras","title":"Detecting Exceptions and Errors in Tests - PropCheck - Property based testing for Elixir","doc":"PropCheck can attempt to detect and output exceptions in non-verbose mode. This can be done using\nthe `detect_exceptions: true` option for `property` or `forall`, or using the environment variable\n`PROPCHECK_DETECT_EXCEPTIONS`. If this environment variable is set to 1, exception detection is enabled\nglobally.","ref":"readme.html#detecting-exceptions-and-errors-in-tests"},{"type":"extras","title":"Links to other documentation - PropCheck - Property based testing for Elixir","doc":"The guides for PropEr are an essential set of documents to make full use of `PropCheck`\n\n* [PropEr Home Page](https://proper-testing.github.io/index.html)\n* [PropEr Tutorials](https://proper-testing.github.io/tutorials.html)\n\nElixir versions of most of PropEr's tutorial material can be found in the\n[test folder on GitHub](https://github.com/alfert/propcheck/tree/master/test).\n\nJesper Andersen and Robert Aloi blog about their thoughts and experiences on\nusing QuickCheck which are (mostly) directly transferable to PropCheck (with\nthe notable exception of concurrency and the new state machine DSL from\nQuickCheck with the possibility to add requirement tags):\n\n* [Jesper Andersen's QuickCheck Advice](https://medium.com/@jlouis666/quickcheck-advice-c357efb4e7e6#.b9wpla7oi)\n* [Jesper Andersen's Breaking Erlang Maps (4 part series)](https://medium.com/@jlouis666/breaking-erlang-maps-4-4ebc3c64068c#.4d61kua92)\n* [Roberto Aloi's Notes on Erlang Quickcheck](http://roberto-aloi.com/erlang/notes-on-erlang-quickcheck)\n\nRather new introductory resources are\n\n* [Fred Hebert's PropEr Testing](http://propertesting.com) and\n* Fred Hebert's book [Property-Based Testing With PropEr, Erlang and Elixir](https://pragprog.com/book/fhproper/property-based-testing-with-proper-erlang-and-elixir).\n  This book explains the new approach of targeted property based testing (TPBT) very nicely\n  and is way more approachable then the scientific papers regarding TPBT. Some of the\n  Erlang only examples of the book are ported to Elixir and can be found in the test\n  [test folder on GitHub](https://github.com/alfert/propcheck/tree/master/test).\n\nIn contrast to the book, the free website is concerned with\nErlang only. The Erlang examples translate easily into Elixir (beside\nthat at least a reading knowledge of Erlang is extremely helpful to survive\nin the BEAM ecosystem ...). Eventually I will port some of the examples to\nElixir and PropCheck and certainly like to accept PRs.","ref":"readme.html#links-to-other-documentation"},{"type":"extras","title":"What is not available - PropCheck - Property based testing for Elixir","doc":"PropCheck does not support PropEr's capability to derive automatically type\ngenerators from type specifications. This is due to some shortcomings in PropEr,\nwhere the specification analysis in certain situation attempt to parse the Erlang source\ncode of the file with the type specification. Apparently, this fails if the\ntypes are specified in an Elixir source file.\n\nEffectively this means, that to\nto support this feature from Elixir, the type management system in PropEr needs\nto be rewritten completely. Jesper Andersen argues in his aforementioned blog\npost eloquently that automatically derived type generators are not needed, even\nmore that carefully crafted type generators for specific testing purposes is\nan essential part of the QuickCheck philosophy. Therefore, this missing feature\nis not that bad. For the same reason, automatic `@spec`-checking is of limited\nvalue in PropCheck since type generators for functions specification are also\ngenerated automatically.\n\nPropCheck has only very limited support for parallel testing since it introduces\nno new features for concurrency compared to PropEr.","ref":"readme.html#what-is-not-available"},{"type":"extras","title":"Contributing - PropCheck - Property based testing for Elixir","doc":"Please use the GitHub issue tracker for\n\n* bug reports and for\n* submitting pull requests","ref":"readme.html#contributing"},{"type":"extras","title":"Test Policy - PropCheck - Property based testing for Elixir","doc":"Before submitting a pull request, please use Credo to ensure code consistency\nand run `mix tests` to check PropCheck itself. `mix tests` is a Mix alias that\nruns regular tests (via `mix test`) and some external tests (via another Mix\nalias `mix test_ext`) which test PropCheck's side effects, like storing\ncounterexamples or proper output format, that can't be easily tested using\nregular tests.","ref":"readme.html#test-policy"},{"type":"extras","title":"Rebase Policy - PropCheck - Property based testing for Elixir","doc":"Ensure that your PR is based on the latest `master` version by rebasing. If your\nfeature branch is `my_feature`, then do the following before publishing a\npull request:\n\n    git checkout master\n    git pull --rebase\n    git checkout my_feature\n    git rebase master\n\nIf your PR implementation takes longer such that another PR is merged before\nyour own PR, then you have to repeat the above sequence. It might be that you\nhave fix some conflicts. But now you cannot push your branch again, because you\nchanged the history of your local branch compared to the one published on GitHub.\nTherefore, you have force-push your branch. Do this with\n\n    git push --force-with-lease\n\nA simple `git push --force` is not allowed, `--force-with-lease` is more friendly\nand thus allowed. See details in the Git documentation.\n\nThe rationale behind this policy is that we want a simple almost linear history,\nwhere each merged PR create a sequence of merge with no parallel work. This history\nwill not show how many active branches are available during development but the\nsequence of incorporating changes to master. That is the important part and we\nwant to see which commit sequence lead to the specific feature. Merges destroy\nthis linearity. But beware, you can do nasty things with `git rebase`, therefore\nstick to the simple procedure explained above to not corrupt your repository.","ref":"readme.html#rebase-policy"},{"type":"extras","title":"License - PropCheck - Property based testing for Elixir","doc":"PropCheck is provided under the GPL 3 License due to its intimate use of PropEr\n(which is licensed under GPL 3). In particular, PropEr's exclusion rule of\nopen source software from copyleft applies here as well [as described in this discussion on GitHub](https://github.com/proper-testing/proper/issues/29#issuecomment-4956226).\n\nPersonally, I would prefer to use the LPGL to differentiate between extending PropEr\nand PropCheck as GPL-licensed software and the use of PropEr/PropCheck, which would\nnot be infected by GPL's copyleft. But as long as PropEr does not change its\nlicensing, we have no options. PropCheck is clearly an extension of PropEr, so it\nis derived work falling under GPL's copyleft. Using LGPL or any other license for\nPropCheck will not help, since GPL's copyleft overrules other licenses or result\nin an invalid or at least questionable licensing which does not help anybody.\n\nFrom my understanding of open source licenses as a legal amateur, the situation is\ncurrently as follows: Since PropCheck is a testing framework, the\nsystem under test is not infected by the copyleft of GPL, since PropCheck is only\na tool used temporarily during development of the system under test. At least,\nif you don't distribute your system together with the test code and the test libs\nas a binary. Another friendly approach is\nto have the tests in a separate project, such that the tests are a real client\nof the system under test. But again, this is my personal take. In question, please\nconsult a professional legal advisor.","ref":"readme.html#license"},{"type":"extras","title":"PropCheck Changelog","doc":"# PropCheck Changelog","ref":"changelog.html"},{"type":"extras","title":"Current Development - PropCheck Changelog","doc":"","ref":"changelog.html#current-development"},{"type":"extras","title":"1.4.2 - PropCheck Changelog","doc":"* Eliminate warnings on dependencies. Thanks to  \n* Correct output for parallel runs, thanks to  \n* Documentation improvements by  ,  ,","ref":"changelog.html#1-4-2"},{"type":"extras","title":"1.4.0 - PropCheck Changelog","doc":"* Support for PropEr 1.4.0","ref":"changelog.html#1-4-0"},{"type":"extras","title":"1.3.1 - PropCheck Changelog","doc":"* Configuration of counter example files for umbrella project. Thanks to [https://github.com/evnu](https://github.com/evnu)\n* Readme and mix optimisation. Thanks to [@kianmeng](https://github.com/kianmeng)","ref":"changelog.html#1-3-1"},{"type":"extras","title":"1.3.0 - PropCheck Changelog","doc":"* Upgrade to Elixir 1.7 as lowest Elixir version, since `get_stacktrace()` is deprecated in Elixir 1.11. Thanks to [@flowerett](https://github.com/flowerett)\n* `:iex` is now an extra-application. Thanks to [@ahamez](https://github.com/ahamez)\n* Support for parallel testing of state machines started. Requires also PRs in the upstream\n  PropEr. An upgrade to (the future) PropEr 1.4 will result in better reports.","ref":"changelog.html#1-3-0"},{"type":"extras","title":"1.2.2 - PropCheck Changelog","doc":"* Fixes to statemachine reported by [@aherranz](https://github.com/aherranz) and [@devonestes](https://github.com/devonestes)\n* Fixes to the Readme linking by [@oo6](https://github.com/oo6)","ref":"changelog.html#1-2-2"},{"type":"extras","title":"1.2.1 - PropCheck Changelog","doc":"* Support for Elixir 1.10 in tests\n* Refactorings of the statemachine implementation to be closer to PropEr. Thanks to [https://github.com/x4lldux](https://github.com/x4lldux).\n* `property/1` for marking properties to be implemented in the future.\n   Thanks to [https://github.com/evnu](https://github.com/evnu)\n* Prevent crashing if no counter examples was returned in a failing property.\n  Thanks to [https://github.com/evnu](https://github.com/evnu)\n* Consistent reporting of Erlang terms in Elixir syntax. Thanks to [https://github.com/x4lldux](https://github.com/x4lldux).\n* Enhanced handling and reporting of exception. Thanks to [https://github.com/evnu](https://github.com/evnu)\n* Include `credo` in the build. Thanks to [https://github.com/evnu](https://github.com/evnu)\n* Fix `PROPCHECK_VERBOSE` to work with `property`\n* Allow `PROPCHECK_VERBOSE=0` to make all properties quiet\n* GitHub Actions are the new CI environemnt.  Thanks to [https://github.com/evnu](https://github.com/evnu)\n* Pinning of variables in `let` allows easier re-use of variables. Thanks to [https://github.com/Ecialo](https://github.com/Ecialo)","ref":"changelog.html#1-2-1"},{"type":"extras","title":"1.2.0 - PropCheck Changelog","doc":"* Handling of tags corrected. This changes slighty existing the behavior and gives\n  reason to introduce a new minor version.\n  Thanks to [https://github.com/evnu](https://github.com/evnu)\n* Verbose settings can be configured at the command line via environment variable\n  `PROPCHECK_VERBOSE`. Thanks to [https://github.com/evnu](https://github.com/evnu)\n* Setting default options to `forall` on module or describe level.\n  Thanks to [https://github.com/x4lldux](https://github.com/x4lldux).\n* Support for Elixir 1.9 in tests. Thanks to [https://github.com/evnu](https://github.com/evnu)\n* Moving back from CircleCI to TravisCI. Thanks to [https://github.com/evnu](https://github.com/evnu)","ref":"changelog.html#1-2-0"},{"type":"extras","title":"1.1.5 - PropCheck Changelog","doc":"* `:verbose` option is propagated from the property directly to `forall`.\n    Thanks to [https://github.com/evnu](https://github.com/evnu)\n* Storing of counter-examples can excluded by tag `:store_counter_example`\n    Thanks to [https://github.com/evnu](https://github.com/evnu)\n* Improved documentation for longer statemachine runs. Thanks\n  to [https://github.com/adkron](https://github.com/adkron)\n* Improved error message for missing command in `StateM.DSL`.\n  Thanks to [https://github.com/devonestes](https://github.com/devonestes)\n* Introduction of linter `credo` and CircleCi as new CI tool.\n  Thanks to [https://github.com/evnu](https://github.com/evnu)\n* `let` syntax is the same now as `forall`.\n  Thanks to [https://github.com/evnu](https://github.com/evnu)","ref":"changelog.html#1-1-5"},{"type":"extras","title":"1.1.4 - PropCheck Changelog","doc":"* Fixes an issue with the setup of regular and targeted properties rendering 1.1.3 unusable\n* Enhanced documentation for targeted properties","ref":"changelog.html#1-1-4"},{"type":"extras","title":"1.1.3 - PropCheck Changelog","doc":"* Better command generator with improved shrinking for complex argument generations.\n* Support for map-generator, thanks to [https://github.com/IRog](https://github.com/IRog)\n* Support for targeted properties, a new feature of Proper 1.3\n* Requires at least Elixir 1.5","ref":"changelog.html#1-1-3"},{"type":"extras","title":"1.1.2 - PropCheck Changelog","doc":"* Proper v1.3.0 is supported (effectively, all 1.x versions are allowed\n  as dependency)","ref":"changelog.html#1-1-2"},{"type":"extras","title":"1.1.1 - PropCheck Changelog","doc":"* the weight callback for the DSL was incorrectly specified and documented. Thanks\n  to [https://github.com/adkron](https://github.com/adkron)","ref":"changelog.html#1-1-1"},{"type":"extras","title":"1.1.0 - PropCheck Changelog","doc":"* New command oriented DSL for testing stateful systems, inspired by EQC and\n  discussions about stateful testing in StreamData\n* More details regarding licensing\n* Rerun of properties fixed\n* Better and corrected type specs, compatible with dialyxir 1.0.0(-rc*)\n* Old modules for automatic type generators removed. They were never completed and\n  since 2016 no longer part of the API (i.e. even before release 0.0.1).","ref":"changelog.html#1-1-0"},{"type":"extras","title":"1.0.6 - PropCheck Changelog","doc":"* After a counter example is resolved, the entire property is run again to\n  ensure that no other counter examples exist. Thanks to [https://github.com/evnu](https://github.com/evnu)\n* tabs vs whitespace corrected for test cases, thanks to [https://github.com/ryanwinchester](https://github.com/ryanwinchester)\n* added a hint about stored counterexamples for users, thanks to [https://github.com/evnu](https://github.com/evnu)\n* Corrected formatting of markdown for documentation, thanks to","ref":"changelog.html#1-0-6"},{"type":"extras","title":"1.0.5 - PropCheck Changelog","doc":"* Allows to use `ExUnit` assertions as boolean conditions, thanks to [https://github.com/evnu](https://github.com/evnu)\n* `let` and `let_shrink` allow more than two parameters, thanks to [https://github.com/BinaryNoggin](https://github.com/BinaryNoggin)\n* Errors, that aren't counter examples, are no longer stored as counter examples,  thanks to [https://github.com/evnu](https://github.com/evnu)\n* new feature `sample_shrink`, thanks to [https://github.com/evnu](https://github.com/evnu)\n* the examples for stateful testing use `GenServer.stop/0` for a reliable\n  stopping of gen servers.\n* several documentation issues","ref":"changelog.html#1-0-5"},{"type":"extras","title":"1.0.4 - PropCheck Changelog","doc":"* `produce` has now a valid default parameter\n* Removed several lazy compiler warnings\n* Link in README corrected.","ref":"changelog.html#1-0-4"},{"type":"extras","title":"1.0.3 - PropCheck Changelog","doc":"* Removed debug log output.","ref":"changelog.html#1-0-3"},{"type":"extras","title":"1.0.2 - PropCheck Changelog","doc":"* only labeled, never released...","ref":"changelog.html#1-0-2"},{"type":"extras","title":"1.0.1 - PropCheck Changelog","doc":"* Bugfix for Mix integration in Umbrella projects, thanks to [https://github.com/evnu](https://github.com/evnu)","ref":"changelog.html#1-0-1"},{"type":"extras","title":"1.0.0 - PropCheck Changelog","doc":"* Counter examples are automatically stored and reapplied until the properties work\n  or the counter examples are deleted. See [https://github.com/alfert/propcheck/pull/18](https://github.com/alfert/propcheck/pull/18)\n* Mix configuration for counter examples file and for inspecting and cleaning\n  counter examples.","ref":"changelog.html#1-0-0"},{"type":"extras","title":"0.0.2 - PropCheck Changelog","doc":"* Fixed a lot of 1.5 (and 1.4) Elixir warnings thanks to [https://github.com/evnu](https://github.com/evnu)\n* Readme additions regarding installation thanks to [https://github.com/evnu](https://github.com/evnu)\n* Added more concurrency robustness for the ping pong tests\n* Fixed a bug a in the movie server, which did not startup properly.","ref":"changelog.html#0-0-2"},{"type":"extras","title":"0.0.1 - PropCheck Changelog","doc":"* Initial release","ref":"changelog.html#0-0-1"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}